/*
 * $Id: clientserver.h,v 1.2 2021/03/29 10:20:29 klapetek Exp $
 *
 * This file provides some common functions for client.c and server.c.
 *
 * I, the copyright holder of this work, release this work into the public
 * domain.  This applies worldwide.  In some countries this may not be legally
 * possible; if so: I grant anyone the right to use this work for any purpose,
 * without any conditions, unless such conditions are required by law.
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <assert.h>
#include <unistd.h>
#include <netdb.h>
#include <netinet/in.h>
#include <sys/types.h>
#include "gwyfile.h"

static void
ensure_buffer_size(char **buffer, size_t *bufsize, size_t len)
{
    if (len <= *bufsize)
        return;

    if (2*(*bufsize) >= len)
        *bufsize *= 2;
    else
        *bufsize = len;

    *buffer = realloc(*buffer, *bufsize);
}

static GwyfileObject*
recv_gwyfile_message(int sock, char **buffer, size_t *bufsize, int *err)
{
    GwyfileError *error = NULL;
    GwyfileObject *message;
    size_t objsize, len = 0;
    ssize_t read_size;

    if (sock==-1) return NULL;

    /* Purposedly make the buffer small at the beginning. */
    if (!*bufsize || !*buffer)
        ensure_buffer_size(buffer, bufsize, 16);

    *err = -1;
    //printf("Trying to read %u bytes.\n", (unsigned)(*bufsize - len));
    while ((read_size = read(sock, *buffer + len, *bufsize - len)) > 0) {
        //printf("Read %ld bytes from socket %d\n", (long int)read_size, sock);
        len += read_size;
        /* Process the message we have in the buffer. */
        if (!(message = gwyfile_object_read_memory(*buffer, len, &objsize, &error))) {
            assert(error);
            if (error->domain == GWYFILE_ERROR_DOMAIN_DATA && error->code == GWYFILE_ERROR_TRUNCATED) {
                /* There is an incomplete message. */
                gwyfile_error_clear(&error);
                ensure_buffer_size(buffer, bufsize, 2*(*bufsize));
                continue;
            }
            else {
                fprintf(stderr, "Failed to read message. %s\n", error->message);
                fprintf(stderr, "Giving up.\n");
                gwyfile_error_clear(&error);
                return NULL;
            }
        }
        //printf("Obtained message of %lu bytes\n", (unsigned long int)objsize);
        *err = 0;
        return message;
    }
    /* If either read_size < 0 or we have still something in buffer then an error occurred. */
    if (read_size < 0) {
        fprintf(stderr, "recv() failed: %s.\n", strerror(errno));
        return NULL;
    }
    else if (len > 0) {
        fprintf(stderr, "Incomplete message.\n");
        return NULL;
    }

    /* Returning NULL with zero error means there is simply nothing more to read. */
    *err = 0;
    return NULL;
}

static int
send_gwyfile_message(GwyfileObject *message, int sock, char **buffer, size_t *bufsize)
{
    ssize_t pos, write_size;
    size_t len;

    if (sock == -1) return -1;

int err = 0;
socklen_t size = sizeof (err);
int check = getsockopt(sock, SOL_SOCKET, SO_ERROR, &err, &size);
if (check != 0) 
{
    printf("krucis\n");  //some code
}

    len = gwyfile_object_size(message);
    ensure_buffer_size(buffer, bufsize, len);
    len = gwyfile_object_write_memory(message, *buffer, *bufsize, NULL);
    assert(len);
    pos = 0;
    while (len > 0) {
        write_size = write(sock, *buffer + pos, len);
        if (write_size < 0) {
            fprintf(stderr, "send() failed: %s.\n", strerror(errno));
            return -1;
        }
        len -= write_size;
        pos += write_size;
    }

    return 0;
}

