
/* GwyScope
 * Copyright (C) 2021 Petr Klapetek
*/

#include <gtk/gtk.h>

#include "gui.h"
#include "hw.h"

#include <app/gwyapp.h>
#include <libgwymodule/gwymodule-file.h>
#include <libgwymodule/gwymodule-layer.h>
#include <libgwydgets/gwystatusbar.h>
#include <libgwydgets/gwydgetutils.h>
#include <libgwydgets/gwycombobox.h>
#include <gwyscan.h>
#include "gwyfile.h"
#include <string.h>
#include "chanview.h"
#include "freqview.h"
#include "laserview.h"
#include "tableview.h"
#include "channels.h"
#include "signals.h"

#define MINISCAN_APP_NAME           "Miniscan PataMat"
#define PREVIEW_STEP_POINTS         1000
#define PREVIEW_CHECK_STEP_POINTS   1000

void create_path(MiniData *md);
gint get_path_length(const MiniData *md);

void ramp(MiniData *md);


void do_save_file(MiniData *md, const gchar *filename)
{
    GwyContainer *all;
    GError *error = NULL;
    gchar buff[256];

    if (!md->c_sc_reverse)
        gwy_file_save(md->container, filename, GWY_RUN_IMMEDIATE, &error);  
    else
    {
        all = gwy_container_duplicate(md->container); //FIXME: add the reverse data
        gwy_file_save(all, filename, GWY_RUN_IMMEDIATE, &error); 
        gwy_object_unref(all);
    }
    
    if (error != NULL) {
        g_snprintf(buff, sizeof(buff), "Error saving data to %s: %s", filename, error->message);
        writelog(md, buff);
        gwy_statusbar_set_markup(GWY_STATUSBAR(md->statusbar_measure), buff);
    }
    else {
        g_snprintf(buff, sizeof(buff), "Data successfully saved to %s", filename);
        writelog(md, buff);
//        gwy_statusbar_set_markup(GWY_STATUSBAR(md->statusbar_measure), buff);
    }
}

void do_save_project(MiniData *md, const gchar *filename)
{
    GError *error = NULL;
    gchar buff[256];

    if (md->project!=NULL) {
       gwy_file_save(md->project, filename, GWY_RUN_IMMEDIATE, &error);
    
       if (error != NULL) {
           g_snprintf(buff, sizeof(buff), "Error saving project to %s: %s", filename, error->message);
           writelog(md, buff);
           gwy_statusbar_set_markup(GWY_STATUSBAR(md->statusbar_measure), buff);
       }
       else {
           g_snprintf(buff, sizeof(buff), "Project successfully saved to %s", filename);
           writelog(md, buff);
//           gwy_statusbar_set_markup(GWY_STATUSBAR(md->statusbar_measure), buff);
       }
    }
}


void do_save_xyz_file(MiniData *md, char* filename)
{
    GwyscanResult result;
    gchar buff[256], rfilename[256]; 
    gint xres, yres;

    printf("saving %d channels as xyz file\n", md->nchan);

   if (md->page==1) { //adaptive
       xres = md->xres;
       yres = md->yres;
    } else {
       xres = md->rxres;
       yres = md->ryres;
    }


    result = gwyscan_save_gwyddion_xyz(md->xypos, md->vdata, md->npos, md->nchan, xres, yres, md->descriptions, filename);
   
    if (result != GWYSCAN_SUCCESS) 
    {       
        g_snprintf(buff, sizeof(buff), "Error saving data to %s", filename);
        writelog(md, buff);
        if (result == GWYSCAN_WRITE_FAILED)
            writelog(md, "gwyscan_save_gwyddion_xyz result: GWYSCAN_WRITE_FAILED");
        if (result == GWYSCAN_ALLOCATION_FAILED)
            writelog(md, "gwyscan_save_gwyddion_xyz result: GWYSCAN_ALLOCATION_FAILED");
        //gtk_statusbar_push(GTK_STATUSBAR(md->statusbar), md->statusbar_context_id, buff);        
        gwy_statusbar_set_markup(GWY_STATUSBAR(md->statusbar_measure), buff);
    }
    else 
    {
        g_snprintf(buff, sizeof(buff), "Data successfully saved to %s", filename);
        writelog(md, buff);
        //gtk_statusbar_push(GTK_STATUSBAR(md->statusbar), md->statusbar_context_id, buff);
    //    gwy_statusbar_set_markup(GWY_STATUSBAR(md->statusbar_measure), buff);
    }


    if (md->c_sc_reverse) {
        g_snprintf(rfilename, sizeof(rfilename), "%s.reverse", filename);

        result = gwyscan_save_gwyddion_xyz(md->rxypos, md->rvdata, md->rnpos, md->nchan, xres, yres, md->descriptions, rfilename);
   
        if (result != GWYSCAN_SUCCESS) 
        {       
            g_snprintf(buff, sizeof(buff), "Error saving reverse data to %s", rfilename);
            writelog(md, buff);
            if (result == GWYSCAN_WRITE_FAILED)
                writelog(md, "gwyscan_save_gwyddion_xyz result: GWYSCAN_WRITE_FAILED");
            if (result == GWYSCAN_ALLOCATION_FAILED)
                writelog(md, "gwyscan_save_gwyddion_xyz result: GWYSCAN_ALLOCATION_FAILED");
            //gtk_statusbar_push(GTK_STATUSBAR(md->statusbar), md->statusbar_context_id, buff);        
        //    gwy_statusbar_set_markup(GWY_STATUSBAR(md->statusbar_measure), buff);
        }
        else 
        {
            g_snprintf(buff, sizeof(buff), "Reverse data successfully saved to %s", rfilename);
            writelog(md, buff);
            //gtk_statusbar_push(GTK_STATUSBAR(md->statusbar), md->statusbar_context_id, buff);
      //      gwy_statusbar_set_markup(GWY_STATUSBAR(md->statusbar_measure), buff);
        }

    }

}

void do_save_brick_file(MiniData *md, char* filename)
{
    GwyContainer *data;
    GError *error = NULL;
    gchar *title;

    if (md->brick_z1==NULL || md->brick_e1==NULL || md->brick_i1==NULL) {
       fprintf(stderr, "No brick to save!\n");
       return;
    }

    printf("Saving brick\n");

    data = gwy_container_new();

    gwy_container_set_object(data, gwy_app_get_brick_key_for_id(0), md->brick_z1);
    title = g_strdup("z approach");
    gwy_container_set_string(data, gwy_app_get_brick_title_key_for_id(0), (const guchar*)title);
 
    gwy_container_set_object(data, gwy_app_get_brick_key_for_id(1), md->brick_z2);
    title = g_strdup("z retract");
    gwy_container_set_string(data, gwy_app_get_brick_title_key_for_id(1), (const guchar*)title);

    gwy_container_set_object(data, gwy_app_get_brick_key_for_id(2), md->brick_e1);
    title = g_strdup("error signal approach");
    gwy_container_set_string(data, gwy_app_get_brick_title_key_for_id(2), (const guchar*)title);

    gwy_container_set_object(data, gwy_app_get_brick_key_for_id(3), md->brick_e2);
    title = g_strdup("error signal retract");
    gwy_container_set_string(data, gwy_app_get_brick_title_key_for_id(3), (const guchar*)title);

    gwy_container_set_object(data, gwy_app_get_brick_key_for_id(4), md->brick_i1);
    title = g_strdup("in2 approach");
    gwy_container_set_string(data, gwy_app_get_brick_title_key_for_id(4), (const guchar*)title);

    gwy_container_set_object(data, gwy_app_get_brick_key_for_id(5), md->brick_i2);
    title = g_strdup("in2 retract");
    gwy_container_set_string(data, gwy_app_get_brick_title_key_for_id(5), (const guchar*)title);


    if (!gwy_file_save(data, filename, GWY_FILE_OPERATION_SAVE, &error))
    {
        fprintf(stderr, "Cannot write %s: %s\n", filename, error->message);
    }
    else printf("Bricks saved to %s\n", filename);
    gwy_object_unref(data);
}


void file_clear_cb(GtkWidget *widget, MiniData *md)
{
    md->ndata = 0; //no measured data
    md->npos = 0; //no allocated points
    if (md->xypos!=NULL) {
            g_free(md->xypos);
    }  
    if (md->vdata!=NULL) {
        g_free(md->vdata);
    }
    md->xypos = NULL;
    md->vdata = NULL;

    md->rndata = 0; //no measured data
    md->rnpos = 0; //no allocated points
    if (md->rxypos!=NULL) {
            g_free(md->rxypos);
    }  
    if (md->rvdata!=NULL) {
        g_free(md->rvdata);
    }
    md->rxypos = NULL;
    md->rvdata = NULL;

    printf("Data cleared\n");  

}

void file_save_as_cb(GtkWidget *widget, MiniData *md)
{

   GtkWidget *dialog;

   dialog = gtk_file_chooser_dialog_new ("Save Gwyddion file as...",
                                      GTK_WINDOW(md->toplevel),
                                      GTK_FILE_CHOOSER_ACTION_SAVE,
                                      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                      GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
                                      NULL);

   gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (dialog), TRUE);

   if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
   {
      md->filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
      do_save_file(md, md->filename);
   }
   gtk_widget_destroy(dialog);

}

void file_save_cb(GtkWidget *widget, MiniData *md)
{

   if (md->filename==NULL) file_save_as_cb(widget, md);
   else {
      do_save_file(md, md->filename);
   }

}

void file_save_xyz_as_cb(GtkWidget *widget, MiniData *md)
{

   GtkWidget *dialog;

   dialog = gtk_file_chooser_dialog_new ("Save Gwyddion XYZ file as...",
                                      GTK_WINDOW(md->toplevel),
                                      GTK_FILE_CHOOSER_ACTION_SAVE,
                                      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                      GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
                                      NULL);

   gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (dialog), TRUE);

   if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
   {
      md->filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
      do_save_xyz_file(md, md->filename);
   }
   gtk_widget_destroy(dialog);

}

void file_save_brick_as_cb(GtkWidget *widget, MiniData *md)
{

   GtkWidget *dialog;
   gchar *filename;

   dialog = gtk_file_chooser_dialog_new ("Save brick to Gwyddion file as...",
                                      GTK_WINDOW(md->toplevel),
                                      GTK_FILE_CHOOSER_ACTION_SAVE,
                                      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                      GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
                                      NULL);

   gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (dialog), TRUE);

   if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
   {
      filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
      do_save_brick_file(md, filename);
   }
   gtk_widget_destroy(dialog);

}

void file_save_xyz_cb(GtkWidget *widget, MiniData *md)
{

   if (md->filename==NULL) 
       file_save_xyz_as_cb(widget, md);
   else {
      do_save_xyz_file(md, md->filename);
   }
}


void project_save_as_cb(GtkWidget *widget, MiniData *md)
{

   GtkWidget *dialog;

   dialog = gtk_file_chooser_dialog_new ("Save project as...",
                                      GTK_WINDOW(md->toplevel),
                                      GTK_FILE_CHOOSER_ACTION_SAVE,
                                      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                      GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
                                      NULL);

   gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (dialog), TRUE);

   if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
   {
      md->project_filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
      do_save_project(md, md->project_filename);
   }
   gtk_widget_destroy(dialog);

}
void project_save_cb(GtkWidget *widget, MiniData *md)
{

   if (md->project_filename==NULL) project_save_as_cb(widget, md);
   else {
      do_save_project(md, md->project_filename);
   }

}

void project_clear_cb(GtkWidget *widget, MiniData *md)
{
    if (md->project != NULL) gwy_object_unref(md->project);
    md->project = gwy_container_new(); 

    md->last_project_index = 0;

    gwy_statusbar_set_markup(GWY_STATUSBAR(md->statusbar_measure), "New project started"); 
    printf("project started\n");
}
void project_add_cb(GtkWidget *widget, MiniData *md)
{
    gint i;
    gchar oldkey[100], newkey[100];
    GwyContainer *copy = gwy_container_duplicate(md->container);  //FIXME, duplicate also rcontainer if it exists

    for (i=0; i<md->nchan; i++) {
       g_snprintf(oldkey, sizeof(oldkey), "/%d/data", i);
       g_snprintf(newkey, sizeof(newkey), "/%d/data", i + md->last_project_index);

//       gwy_container_transfer (md->container, md->project, oldkey, newkey, TRUE);
       gwy_container_set_object_by_name(md->project, newkey, gwy_container_get_object_by_name(copy, oldkey));
       printf("transferrred %s to %s\n", oldkey, newkey);

       g_snprintf(oldkey, sizeof(oldkey), "/%d/data/title", i);
       g_snprintf(newkey, sizeof(newkey), "/%d/data/title", i + md->last_project_index);

//       gwy_container_transfer(md->container, md->project, oldkey, newkey, TRUE);
       gwy_container_set_string_by_name(md->project, newkey, (guchar *)g_strdup((gchar *)gwy_container_get_string_by_name(copy, oldkey)));
       printf("transferrred title %s (%s) to %s\n", oldkey, gwy_container_get_string_by_name(copy, oldkey), newkey);
    }

    md->last_project_index += md->nchan;

    gwy_statusbar_set_markup(GWY_STATUSBAR(md->statusbar_measure), "Measured data added to project"); 
    printf("items added\n");

    gwy_object_unref(copy);
}

static void
show_points(MiniData *md)
{
    gdouble xypos[2];
    gint i, npos = 1;

    if (md->in_setup) return;

    if (md->page == 3) {
         xypos[0] = gtk_adjustment_get_value(GTK_ADJUSTMENT(md->ramp_xpos_adj))*1e-6;
         xypos[1] = gtk_adjustment_get_value(GTK_ADJUSTMENT(md->ramp_ypos_adj))*1e-6;

         for (i=0; i<4; i++) {
             gws_chanview_showpath(md->view[i], xypos, npos);
         }
    }
    else {
         for (i=0; i<4; i++)
             gws_chanview_hidepath(md->view[i]);
    }
}

static void
scan_page_switched_cb(MiniData *md,
              G_GNUC_UNUSED GtkNotebookPage *page,
              gint pagenum)


{
    md->page = pagenum;
    show_points(md);

    if (!md->in_setup) {
       if (md->page==3) 
       {
          gws_changraph_change_state(md->graph[0], CHGR_STATE_RAMP);
          gws_changraph_change_state(md->graph[1], CHGR_STATE_RAMP);
       } 
       else
       {
          gws_changraph_change_state(md->graph[0], CHGR_STATE_FREERUNNING);
          gws_changraph_change_state(md->graph[1], CHGR_STATE_FREERUNNING);
       }
   }

}

/*check if approaching in another thread was finished already and if yes, update controls*/
static gboolean
approachupdate(gpointer data)
{
    gboolean approaching;
    MiniData *md = (MiniData *)data;

    pthread_mutex_lock(md->approach_mutex);
    approaching = md->approaching;
    pthread_mutex_unlock(md->approach_mutex);

    if (approaching) return TRUE;
    
    md->state = GWS_NOTHING;
    gtk_button_set_label(GTK_BUTTON(md->button_approach), "Approach");
    gtk_widget_set_sensitive(md->button_tipup, TRUE);
    gtk_widget_set_sensitive(md->button_tipdown, TRUE);
    gtk_widget_set_sensitive(md->button_retract, TRUE);
   
    writelog(md, "approach finished");
 
    return FALSE;
}

/*prepared for arbitrary scanning, however forced to do regular forward scan for now*/
gpointer
approachloop(gpointer data)
{
    int i, j, approached;
    gboolean approaching;
    double zpiezo, error, in1, in2, time, a1, p1;
    MiniData *md = (MiniData *)data;

    pthread_mutex_lock(md->approach_mutex);
    md->approaching = TRUE;
    pthread_mutex_unlock(md->approach_mutex);

    approached = 0;

     for (i=0; i<10000; i++) {   
 
        /*search for feedback*/
        get_actual_zdata(md, &zpiezo, &error, &in1, &in2, &time, &a1, &p1);

        if (error<md->threshold || zpiezo<5.0) {
             approached = 1;
             break;
        }

        //check md->approaching state for getting false, which means user killed it
        pthread_mutex_lock(md->approach_mutex);
        approaching = md->approaching;
        pthread_mutex_unlock(md->approach_mutex);

	if (approaching == FALSE) {
            approached = 0;
            break;
	}

        if (approached == 1 || approaching==FALSE) break;

        /* no feedback was reached, go back with piezo and step down by motor*/
        printf("step down\n");

        for (j=0; j<1; j++) {
           set_motor_down(md);
           usleep(300000);

            //check md->approaching state for getting false, which means user killed it
            pthread_mutex_lock(md->approach_mutex);
            approaching = md->approaching;
            pthread_mutex_unlock(md->approach_mutex);

	    if (approaching == FALSE) {
		    approached = 0;
		    break;
	    }

        }

    }
    if (approached == 0) fprintf(stderr, "Approach failed\n");

    pthread_mutex_lock(md->approach_mutex);
    md->approaching = FALSE;
    pthread_mutex_unlock(md->approach_mutex);
  
    return NULL;
}


void approach_cb(GtkWidget *widget, MiniData *md)
{

    gtk_widget_set_sensitive(GTK_WIDGET(md->zpiezo_spin), FALSE);
    set_feedback(md, TRUE);   

   /*
    if (md->state == GWS_APPROACHING) 
    {  
        md->state = GWS_NOTHING;
        md->approaching = FALSE;   //force loop to exit
        gtk_button_set_label(GTK_BUTTON(widget), "Approach");
        gtk_widget_set_sensitive(md->button_tipup, TRUE);
        gtk_widget_set_sensitive(md->button_tipdown, TRUE);
        gtk_widget_set_sensitive(md->button_retract, TRUE);
    } else {
        md->state = GWS_APPROACHING;
        gtk_button_set_label(GTK_BUTTON(widget), "Stop");
        gtk_widget_set_sensitive(md->button_tipup, FALSE);
        gtk_widget_set_sensitive(md->button_tipdown, FALSE);
        gtk_widget_set_sensitive(md->button_retract, FALSE);
        writelog(md, "approach pressed");

        //sweep with piezo

        //go back with piezo

        //do single motor shift



        start approaching
        //md->approach_loop = g_thread_new("miniscan_approach",
        //                         approachloop,
        //                         md);

        //wait until approach is completed
        //g_timeout_add(500, approachupdate, md);

   
    }*/
}
void retract_cb(GtkWidget *widget, MiniData *md)
{
    gdouble buf, zpiezo;

    set_feedback(md, FALSE);
    set_fmfeedback(md, FALSE);

    get_actual_zdata(md, &zpiezo, &buf, &buf, &buf, &buf, &buf, &buf);
    gtk_widget_set_sensitive(GTK_WIDGET(md->zpiezo_spin), TRUE);
    gtk_adjustment_set_value(GTK_ADJUSTMENT(md->zpiezo_adj), zpiezo);

    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(md->amfm_check), FALSE);

    //if (md->mode==4) //akiyama test? 
    //{   
    //    set_freq(md, 2, md->f*1000, md->exc2, 0); //from kHz to Hz
    //    printf("Frequency set to %g kHz\n", md->f);
    //}


    /*
    gint i;

    for (i=0; i<100; i++) {
       set_motor_up(md);
       usleep(10000);
    }

    usleep(100000);        
    table_move_to(md, 0, 0);
    */
 }

void tipup_cb(GtkWidget *widget, MiniData *md)
{
    gint i;

    for (i=0; i<2; i++) {
        set_motor_up(md);
       usleep(10000);
    }

}

void tipdown_cb(GtkWidget *widget, MiniData *md)
{
    gint i;

    for (i=0; i<2; i++) {
        set_motor_down(md);
       usleep(10000);
    }
}

void hw_reset_time_cb(GtkWidget *widget, MiniData *md)
{    
    reset_start_time(md);

    // get hwserver start time
    gdouble hw_start_time;
    if (get_hw_start_time(md, &hw_start_time))
    {
        pthread_mutex_lock(md->hw_start_time_mutex);
        md->hw_start_time = hw_start_time;
        pthread_mutex_unlock(md->hw_start_time_mutex);
    }
}

void hw_get_version_cb(GtkWidget *widget, MiniData *md)
{    

    printf("version: %s\n", get_version(md));
}

void scan_widgets_read(MiniData *md) 
{
    pthread_mutex_lock(md->scan_mutex);

    gint i, npos;
    gchar buffer[200];

    md->xres = gtk_adjustment_get_value(GTK_ADJUSTMENT(md->xres_adj));
    md->yres = gtk_adjustment_get_value(GTK_ADJUSTMENT(md->yres_adj));
    md->xreal = gtk_adjustment_get_value(GTK_ADJUSTMENT(md->xreal_adj)) * 1e-6; //values were in um
    md->yreal = gtk_adjustment_get_value(GTK_ADJUSTMENT(md->yreal_adj)) * 1e-6; //values were in um
    md->xoff = gtk_adjustment_get_value(GTK_ADJUSTMENT(md->xoff_adj)) * 1e-6; //values were in um
    md->yoff = gtk_adjustment_get_value(GTK_ADJUSTMENT(md->yoff_adj)) * 1e-6; //values were in um
    md->rotate = gtk_adjustment_get_value(GTK_ADJUSTMENT(md->rotate_adj));
    md->profile = gtk_adjustment_get_value(GTK_ADJUSTMENT(md->profile_adj));
    md->profileonly = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(md->profile_check));
    md->rxres = gtk_adjustment_get_value(GTK_ADJUSTMENT(md->rxres_adj));
    md->ryres = gtk_adjustment_get_value(GTK_ADJUSTMENT(md->ryres_adj));
    md->rxreal = gtk_adjustment_get_value(GTK_ADJUSTMENT(md->rxreal_adj)) * 1e-6; //values were in um
    md->ryreal = gtk_adjustment_get_value(GTK_ADJUSTMENT(md->ryreal_adj)) * 1e-6; //values were in um
    md->rxoff = gtk_adjustment_get_value(GTK_ADJUSTMENT(md->rxoff_adj)) * 1e-6; //values were in um
    md->ryoff = gtk_adjustment_get_value(GTK_ADJUSTMENT(md->ryoff_adj)) * 1e-6; //values were in um
    md->rrotate = gtk_adjustment_get_value(GTK_ADJUSTMENT(md->rrotate_adj));
    md->rprofile = gtk_adjustment_get_value(GTK_ADJUSTMENT(md->rprofile_adj));
    md->rprofileonly = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(md->rprofile_check));
    md->nvals = gtk_adjustment_get_value(GTK_ADJUSTMENT(md->nvals_adj));
    md->order = gtk_adjustment_get_value(GTK_ADJUSTMENT(md->order_adj));
    md->nprofiles = gtk_adjustment_get_value(GTK_ADJUSTMENT(md->nprofiles_adj));
    md->noctaves = gtk_adjustment_get_value(GTK_ADJUSTMENT(md->noctaves_adj));
    md->wait = gtk_adjustment_get_value(GTK_ADJUSTMENT(md->wait_adj));
    md->speed = gtk_adjustment_get_value(GTK_ADJUSTMENT(md->speed_adj));
    md->preview_step = gtk_adjustment_get_value(GTK_ADJUSTMENT(md->preview_step_adj));
    md->ramp_zres = gtk_adjustment_get_value(GTK_ADJUSTMENT(md->ramp_zres_adj));
    md->ramp_zfrom = gtk_adjustment_get_value(GTK_ADJUSTMENT(md->ramp_zfrom_adj));
    md->ramp_zto = gtk_adjustment_get_value(GTK_ADJUSTMENT(md->ramp_zto_adj));
    md->ramp_xpos = gtk_adjustment_get_value(GTK_ADJUSTMENT(md->ramp_xpos_adj))*1e-6;
    md->ramp_ypos = gtk_adjustment_get_value(GTK_ADJUSTMENT(md->ramp_ypos_adj))*1e-6;
    md->ramp_time = gtk_adjustment_get_value(GTK_ADJUSTMENT(md->ramp_time_adj));
    md->ramp_starttime = gtk_adjustment_get_value(GTK_ADJUSTMENT(md->ramp_starttime_adj));
    md->ramp_peaktime = gtk_adjustment_get_value(GTK_ADJUSTMENT(md->ramp_peaktime_adj));


    for (i=0; i<4; i++) {
	    gws_chanview_set_ranges(md->view[i], md->xreal, md->yreal);
    }
   

    gtk_adjustment_set_upper(GTK_ADJUSTMENT(md->ramp_xpos_adj), md->xreal/1e-6);
    gtk_adjustment_set_upper(GTK_ADJUSTMENT(md->ramp_ypos_adj), md->yreal/1e-6);

    if (md->ramp_xpos > md->xreal) {
       md->ramp_xpos = md->xreal;
       gtk_adjustment_set_value(GTK_ADJUSTMENT(md->ramp_xpos_adj), md->xreal/1e-6);
    }
    if (md->ramp_ypos > md->yreal) {
       md->ramp_ypos = md->yreal;
       gtk_adjustment_set_value(GTK_ADJUSTMENT(md->ramp_ypos_adj), md->yreal/1e-6);
    }

    show_points(md);

    //create_path(md);
    npos = get_path_length(md);
    g_snprintf(buffer, sizeof(buffer), "Path length: %d points", npos);
    gtk_label_set_text(GTK_LABEL(md->pathlabel), buffer);

    pthread_mutex_unlock(md->scan_mutex);
}

void scan_widgets_activate(MiniData *md, gboolean activate) 
{
    gtk_widget_set_sensitive(md->xres_spin, activate);
    gtk_widget_set_sensitive(md->yres_spin, activate);
    gtk_widget_set_sensitive(md->xreal_spin, activate);
    gtk_widget_set_sensitive(md->yreal_spin, activate);
    gtk_widget_set_sensitive(md->xoff_spin, activate);
    gtk_widget_set_sensitive(md->yoff_spin, activate);
    gtk_widget_set_sensitive(md->profile_spin, activate);
    gtk_widget_set_sensitive(md->rotate_spin, activate);
    gtk_widget_set_sensitive(md->profile_check, activate);
    gtk_widget_set_sensitive(md->nvals_spin, activate);
    gtk_widget_set_sensitive(md->order_spin, activate);
    gtk_widget_set_sensitive(md->nprofiles_spin, activate);
    gtk_widget_set_sensitive(md->noctaves_spin, activate);
}

void save_backup(MiniData *md)
{
    //gchar *filename_gwy;
    gchar *filename_gxyzf;
    gchar *filename_brick;

    //filename_gwy = g_build_filename(g_get_home_dir(), MINISCAN_INI_FOLDER, "miniscan_backup.gwy", NULL);
    filename_gxyzf = g_build_filename(g_get_home_dir(), MINISCAN_INI_FOLDER, "miniscan_backup.gxyzf", NULL);
    filename_brick = g_build_filename(g_get_home_dir(), MINISCAN_INI_FOLDER, "miniscan_backup_brick.gwy", NULL);

    pthread_mutex_lock(md->dfield_mutex);

    //gwy_file_save(md->container, filename_gwy, GWY_RUN_IMMEDIATE, NULL);
    //do_save_file(md, filename_gwy);
    do_save_xyz_file(md, filename_gxyzf);
    do_save_brick_file(md, filename_brick);

    pthread_mutex_unlock(md->dfield_mutex);

    //g_free(filename_gwy);
    g_free(filename_gxyzf);
    g_free(filename_brick);
}

void 
preview_run(MiniData *md, gdouble *xypos, gint npos, gdouble *vdata, gint ndata, GwyDataField **dfield)
{
    gint k;
    gint pxres, pyres, xres, yres;
    gdouble xreal, yreal, xoff, yoff, rotate;
    gdouble *mdata = NULL;

    xres = yres = PREVIEW_SIZE_PIXELS;

    pxres = gwy_data_field_get_xres(dfield[0]);
    pyres = gwy_data_field_get_yres(dfield[0]);

    if (md->page==1) { //adaptive
       xreal = md->xreal;
       yreal = md->yreal;
       xoff = md->xoff;
       yoff = md->yoff;
       rotate = md->rotate;
    } else {
       xreal = md->rxreal;
       yreal = md->ryreal;
       xoff = md->rxoff;
       yoff = md->ryoff;
       rotate = md->rrotate;
    }

    if (ndata>0) {

       for (k=0; k<md->nchan; k++) {
          md->preview_df[k] = gwy_data_field_new(xres, yres, xreal, yreal, 0);
          md->preview_data[k] = (gdouble *)gwy_data_field_get_data_const(md->preview_df[k]);
       }

       //xoff = yoff = 0;
//       printf("running preview on %d channels\n", md->nchan);
       gwyscan_preview_subset_average_all_channels(xypos, vdata, npos, 0, ndata, md->nchan, md->preview_data, mdata, xres, yres, xreal, yreal, xoff, yoff, rotate);
    
       for (k=0; k<md->nchan; k++) {
          gwy_data_field_resample(md->preview_df[k], pxres, pyres, GWY_INTERPOLATION_BILINEAR);

          gwy_data_field_copy(md->preview_df[k], dfield[k], 0);  
          gwy_data_field_data_changed(dfield[k]);

          g_object_unref(md->preview_df[k]);
       }
    } else {
       for (k=0; k<md->nchan; k++) gwy_data_field_fill(dfield[k], 0);
    }
     
}
 
void preview(MiniData *md)
{
    pthread_mutex_lock(md->dfield_mutex);
    pthread_mutex_lock(md->rdfield_mutex);
    gws_changraph_set_running(md->graph[0], FALSE);
    gws_changraph_set_running(md->graph[1], FALSE);

    preview_run(md, md->xypos, md->npos, md->vdata, md->ndata, md->dfield);
    if (md->c_sc_reverse) {
       preview_run(md, md->rxypos, md->rnpos, md->rvdata, md->rndata, md->rdfield);
    }

    md->actual_profile = md->actual_profile_candidate;

    pthread_mutex_unlock(md->dfield_mutex);
    pthread_mutex_unlock(md->rdfield_mutex);
    gws_changraph_set_running(md->graph[0], TRUE);
    gws_changraph_set_running(md->graph[1], TRUE);
}

//run scan line by line
gpointer
scanloop_line(gpointer data)
{    
    MiniData *md = (MiniData *)data;
    gint i, j, k,nprofile;
    gint npos, xres, yres, ndata, lndata, n, rn, skip, report;
    gint preview_step = PREVIEW_STEP_POINTS;
    gdouble xreal, yreal, xoff, yoff, *liftzdata = NULL;
    gboolean scanning;
    gchar buff[100];
    gint nstart, nprev, liftn[5];

    pthread_mutex_lock(md->scan_mutex);
    xres = md->rxres;
    yres = md->ryres;
    xreal = md->rxreal;
    yreal = md->ryreal;
    xoff = md->rxoff;
    yoff = md->ryoff;
    md->scan_start_time = md->time_actual;
    md->scan_end_time = 0;
    pthread_mutex_unlock(md->scan_mutex);

    get_actual_preview_step(md, &preview_step);

    writelog_date(md);
    g_snprintf(buff, sizeof(buff), "starting scanning through %dx%d positions", xres, yres);
    writelog(md, buff);

    npos = xres*yres;
    skip = npos/20;
    report = npos/10;
   
    printf("starting line scanning thread, npos = %d\n", npos); 

    scanning = TRUE;

    n = rn = liftn[0] = liftn[1] = liftn[2] = liftn[3] = liftn[4] = nprev = 0;

    if (md->nlift) liftzdata = (gdouble *)malloc(xres*sizeof(gdouble));

    for (j=0; j<yres; j++)
    { 
       pthread_mutex_lock(md->scan_mutex);
       scanning = md->scanning;
       pthread_mutex_unlock(md->scan_mutex);

       if (scanning==FALSE) {
            printf("scan stopped\n");
            break;
       } 

       /*update present profile index to show in graph*/
       pthread_mutex_lock(md->dfield_mutex);
       if (j>1) md->actual_profile_candidate = (gint)((gdouble)PREVIEW_SIZE_PIXELS*((gdouble) j)/((gdouble)yres)-1);
       else md->actual_profile_candidate = 0;

       if (md->actual_profile_candidate<0) md->actual_profile_candidate = 0;
       if (md->actual_profile_candidate>=PREVIEW_SIZE_PIXELS) md->actual_profile_candidate = PREVIEW_SIZE_PIXELS-1;

       pthread_mutex_unlock(md->dfield_mutex);

       //printf("scanning forward line %d at %g\n", j, yoff + yreal*(gdouble)j/((gdouble)yres));
       //forward line
       run_scan_line(md, 
                     xoff, 
                     yoff + yreal*(gdouble)j/((gdouble)yres), 
                     xoff + xreal, 
                     yoff + yreal*(gdouble)j/((gdouble)yres),
                     xres, NULL);
 
       nprofile = ndata = lndata = 0;
       do {
         /*get data periodically and add them to result*/
         ndata = get_scan_ndata(md); //0..500

         pthread_mutex_lock(md->scan_mutex);
         scanning = md->scanning;
         pthread_mutex_unlock(md->scan_mutex);

         if (scanning==FALSE) {
              printf("scan stopped\n");
              break;
         } 

         if ((ndata-lndata)>0) {
             update_scan_path_data(md, md->vdata, &md->ndata, md->npos, lndata, ndata, n, 0, NULL);
             n += ndata-lndata;
             nprofile += ndata-lndata;
             nprev += ndata-lndata;
             lndata = ndata;
         }

       } while (nprofile<xres);


       //fill lift z data if needed, moved to lift cycle
/*       if (md->nlift) {
          nstart = 2*md->npos + j*xres; //start of the last forward profile in the flat field vdata
          printf("fill lift: j %d xres %d npos %d ndata %d nstart %d value[nstart]: %g\n", j, xres, md->npos, md->ndata, nstart, md->vdata[nstart]);
          for (i=0; i<xres; i++) {
            liftzdata[i] = md->vdata[nstart + i] - md->lift1_height;
            printf("z %g  ls %g\n", md->vdata[nstart + i], liftzdata[i]);
          }
       }
*/
       //backward line
       if (md->c_sc_reverse) {
          run_scan_line(md, 
                        xoff + xreal, 
                        yoff + yreal*(gdouble)j/((gdouble)yres), 
                        xoff, 
                        yoff + yreal*(gdouble)j/((gdouble)yres),
                        xres, NULL);

          nprofile = ndata = lndata = 0;
          do {
            /*get data periodically and add them to result*/
            ndata = get_scan_ndata(md); //0..500

            pthread_mutex_lock(md->scan_mutex);
            scanning = md->scanning;
            pthread_mutex_unlock(md->scan_mutex);
 
            if (scanning==FALSE) {
                printf("scan stopped\n");
                 break;
            } 

            if ((ndata-lndata)>0) {
                update_scan_path_data(md, md->rvdata, &md->rndata, md->rnpos, lndata, ndata, rn, 0, NULL);
                rn += ndata-lndata;
                nprofile += ndata-lndata;
                lndata = ndata;
            }
          } while (nprofile<xres);
       }

       //lift mode, if any
       if (md->nlift) {
           printf("lift mode!\n");

           for (k=0; k<md->nlift; k++)
           {
              nstart = 2*md->npos + j*xres; //start of the last forward profile in the flat field vdata
              printf("fill lift nstar: lift %d row %d xres %d npos %d ndata %d nstart %d value[nstart]: %g\n", 
                         k, j, xres, md->npos, md->ndata, nstart, md->vdata[nstart]);
              for (i=0; i<xres; i++) {
                 if (k==0) liftzdata[i] = md->vdata[nstart + i] - md->lift1_height;
                 else if (k==1) liftzdata[i] = md->vdata[nstart + i] - md->lift2_height;
                 else if (k==2) liftzdata[i] = md->vdata[nstart + i] - md->lift3_height;
                 else if (k==3) liftzdata[i] = md->vdata[nstart + i] - md->lift4_height;
                 else if (k==4) liftzdata[i] = md->vdata[nstart + i] - md->lift5_height;
                 printf("z[%d] %g  ls %g\n", k, md->vdata[nstart + i], liftzdata[i]);
              }
               //lift mode forward line
              run_scan_line(md, 
                     xoff, 
                     yoff + yreal*(gdouble)j/((gdouble)yres), 
                     xoff + xreal, 
                     yoff + yreal*(gdouble)j/((gdouble)yres),
                     xres,
                     liftzdata); 

              nprofile = ndata = lndata = 0;
              do {
                 /*get data periodically and add them to result*/
                 ndata = get_scan_ndata(md); //0..500
                 pthread_mutex_lock(md->scan_mutex);
                 scanning = md->scanning;
                 pthread_mutex_unlock(md->scan_mutex);
 
                 if (scanning==FALSE) {
                    printf("scan stopped\n");
                    break;
                 } 
 
                 if ((ndata-lndata)>0) {
                    update_scan_path_data(md, md->vdata, &md->ndata, md->npos, lndata, ndata, liftn[k], k+1, NULL); //k=0 would mean no lift
                    liftn[k] += ndata-lndata;
                    nprofile += ndata-lndata;
                    lndata = ndata;
                }
              } while (nprofile<xres);
           }
       }

       //all scanning done
       if (nprev > preview_step) //update at earliest moment after the preview nuber was reached, but not within profile
       {
           get_actual_preview_step(md, &preview_step);
           nprev = 0;
           preview(md);
       }

       if (npos >= 1000)
       {
            if ((n%skip) == 0)
                save_backup(md);
            if ((n%report) == 0)
            {
                g_snprintf(buff, sizeof(buff), "%d%% positions completed (%d)", n / report * 10, npos);
                writelog(md, buff);
            }
       }

       usleep(100000);
    }

    preview(md);

    /*return actual profile back to zero*/
    pthread_mutex_lock(md->dfield_mutex);
    md->actual_profile = md->actual_profile_candidate = 0;
    pthread_mutex_unlock(md->dfield_mutex);

    /*move back to offset values*/
    table_move_to(md, md->xoff, md->yoff);

    pthread_mutex_lock(md->scan_mutex);
    md->scanning = FALSE;
    pthread_mutex_unlock(md->scan_mutex);

    if (md->nlift) g_free(liftzdata);

    return NULL;
}


//scan command was already issued, get the data periodically and check the finish.
gpointer
scanloop_adaptive(gpointer data)
{    
    MiniData *md = (MiniData *)data;
    gint npos, ndata, lndata, n = 0, skip, report;
    gint preview_step = PREVIEW_STEP_POINTS;
    gboolean scanning, creating_path;
    gchar buff[100];
    gint nprev;

    /*wait until creating scan path is completed*/
    do {                
        pthread_mutex_lock(md->create_path_mutex);
        creating_path = md->creating_path;
        pthread_mutex_unlock(md->create_path_mutex);        
        usleep(1000000);
    } while (creating_path == TRUE);    

    pthread_mutex_lock(md->scan_mutex);
    npos = md->npos;
    md->scan_start_time = md->time_actual;
    md->scan_end_time = 0;
    pthread_mutex_unlock(md->scan_mutex);

    get_actual_preview_step(md, &preview_step);

    writelog_date(md);
    g_snprintf(buff, sizeof(buff), "starting scanning through %d positions", npos);
    writelog(md, buff);

    skip = npos/20;
    report = npos/10;
   
    printf("starting adaptive scanning thread, npos = %d\n", npos); 

    scanning = TRUE;
    n = nprev = lndata = ndata = 0;
    do {

        /*get data periodically and add them to result*/
        ndata = get_scan_ndata(md);

        if ((ndata-lndata)>50 || (md->npos-n)<50) {
            update_scan_path_data(md, md->vdata, &md->ndata, md->npos, lndata, ndata, n, 0, NULL);
            n += ndata-lndata;
            nprev += ndata-lndata;
            lndata = ndata;
        }

        //all completed, finish the scan
        if (n == md->npos) scanning = FALSE;
 
        if (nprev > preview_step) //update at earliest moment after the preview nuber was reached
        {
            get_actual_preview_step(md, &preview_step);
            nprev = 0;
            preview(md);
        }

        /*update present profile index to show in graph and check if we were not stopped*/
        pthread_mutex_lock(md->scan_mutex);
        pthread_mutex_lock(md->dfield_mutex);
        scanning = md->scanning;
        if (md->profileonly) 
            md->actual_profile_candidate = (gint)(gwy_data_field_rtoj(md->dfield[0], md->ypos_actual-md->yoff)+ 0.5);    //if single profile is measured, show it
        else 
            md->actual_profile_candidate = MAX(0,((gint)(gwy_data_field_rtoj(md->dfield[0], md->ypos_actual-md->yoff)+ 0.5)-1));    //if normal scan is done, show last complete profile

        pthread_mutex_unlock(md->dfield_mutex);
        pthread_mutex_unlock(md->scan_mutex);

        if (npos >= 1000)
        {
            if ((n%skip) == 0)
                save_backup(md);
            if ((n%report) == 0)
            {
                g_snprintf(buff, sizeof(buff), "%d%% positions completed (%d)", n / report * 10, npos);
                writelog(md, buff);
            }
        }
        usleep(100000);

    } while (n<npos && scanning==TRUE);

    if (npos < 1000)
    {
        save_backup(md);
        g_snprintf(buff, sizeof(buff), "%d%% positions completed (%d)", 100, npos);
        writelog(md, buff);
    }

    preview(md);

    /*return actual profile back to zero*/
    pthread_mutex_lock(md->dfield_mutex);
    md->actual_profile = md->actual_profile_candidate = 0;
    pthread_mutex_unlock(md->dfield_mutex);

    /*move back to offset values*/
    table_move_to(md, md->xoff, md->yoff);

    pthread_mutex_lock(md->scan_mutex);
    md->scanning = FALSE;
    pthread_mutex_unlock(md->scan_mutex);


    return NULL;
}

gpointer
scanloop_ramp(gpointer data)
{    
    MiniData *md = (MiniData *)data;
    gint npos, i, k, ixpos, iypos;
    gint preview_step = PREVIEW_STEP_POINTS;
    gboolean creating_path;
    gdouble zpiezo, error, in1, in2, a1, p1, timestamp;

    gchar buff[100];
    gint nprev;

    /*wait until creating scan path is completed*/
    do {                
        pthread_mutex_lock(md->create_path_mutex);
        creating_path = md->creating_path;
        pthread_mutex_unlock(md->create_path_mutex);        
        usleep(1000000);
    } while (creating_path == TRUE);    

    pthread_mutex_lock(md->scan_mutex);
    npos = md->npos;
    md->scan_start_time = md->time_actual;
    md->scan_end_time = 0;
    pthread_mutex_unlock(md->scan_mutex);

    get_actual_preview_step(md, &preview_step);

    writelog_date(md);
    g_snprintf(buff, sizeof(buff), "starting scanning through %d positions", npos);
    writelog(md, buff);
   
    printf("starting adaptive scanning thread, npos = %d\n", npos); 

    if (md->brick_z1) gwy_object_unref(md->brick_z1);
    if (md->brick_e1) gwy_object_unref(md->brick_e1);
    if (md->brick_i1) gwy_object_unref(md->brick_i1);

    if (md->brick_z2) gwy_object_unref(md->brick_z2);
    if (md->brick_e2) gwy_object_unref(md->brick_e2);
    if (md->brick_i2) gwy_object_unref(md->brick_i2);

    md->brick_z1 = gwy_brick_new(md->rxres, md->ryres, md->ramp_zres, md->xreal, md->yreal, md->ramp_zto - md->ramp_zfrom, TRUE);
    md->brick_e1 = gwy_brick_new(md->rxres, md->ryres, md->ramp_zres, md->xreal, md->yreal, md->ramp_zto - md->ramp_zfrom, TRUE);
    md->brick_i1 = gwy_brick_new(md->rxres, md->ryres, md->ramp_zres, md->xreal, md->yreal, md->ramp_zto - md->ramp_zfrom, TRUE);

    md->brick_z2 = gwy_brick_new(md->rxres, md->ryres, md->ramp_zres, md->xreal, md->yreal, md->ramp_zto - md->ramp_zfrom, TRUE);
    md->brick_e2 = gwy_brick_new(md->rxres, md->ryres, md->ramp_zres, md->xreal, md->yreal, md->ramp_zto - md->ramp_zfrom, TRUE);
    md->brick_i2 = gwy_brick_new(md->rxres, md->ryres, md->ramp_zres, md->xreal, md->yreal, md->ramp_zto - md->ramp_zfrom, TRUE);

    md->scanning = TRUE;
    nprev = 0;
    get_actual_preview_step(md, &preview_step);
    for (i=0; i<md->npos; i++)
    {
        iypos = i/md->rxres;
        ixpos = i - iypos*md->rxres;
             
        printf("move to %d %d  at %g %g\n", ixpos, iypos, md->xypos[2*i], md->xypos[2*i+1]);
        table_move_to(md, md->xypos[2*i], md->xypos[2*i+1]);

        printf("store actual values\n");
        get_actual_zdata(md, &zpiezo, &error, &in1, &in2, &timestamp, &a1, &p1);

        md->vdata[i] = md->xypos[2*i];
        md->vdata[md->npos + i] = md->xypos[2*i+1];
        md->vdata[2*md->npos + i] = zpiezo;
        md->vdata[3*md->npos + i] = error;
        md->vdata[4*md->npos + i] = timestamp;
        md->ndata++;

        printf("ramp\n");
        ramp(md);

        for (k=0; k<md->ramp_zres; k++) {

           gwy_brick_set_val(md->brick_z1, ixpos, iypos, k, md->rampvdata[0*md->ramp_npos + k]);
           gwy_brick_set_val(md->brick_e1, ixpos, iypos, k, md->rampvdata[3*md->ramp_npos + k]);
           gwy_brick_set_val(md->brick_i1, ixpos, iypos, k, md->rampvdata[4*md->ramp_npos + k]);

           gwy_brick_set_val(md->brick_z2, ixpos, iypos, k, md->rampvdata[0*md->ramp_npos + k + md->ramp_zres]);
           gwy_brick_set_val(md->brick_e2, ixpos, iypos, k, md->rampvdata[3*md->ramp_npos + k + md->ramp_zres]);
           gwy_brick_set_val(md->brick_i2, ixpos, iypos, k, md->rampvdata[4*md->ramp_npos + k + md->ramp_zres]);
        }

        usleep(100000); //wait after ramp is finished and feedback on

        if (nprev > preview_step) //update at earliest moment after the preview nuber was reached
        {
            get_actual_preview_step(md, &preview_step);
            nprev = 0;
            preview(md);
        } else nprev++;

        if (md->scanning == FALSE) break;
            
        pthread_mutex_lock(md->dfield_mutex);      
        md->actual_profile_candidate = MAX(0,((gint)(gwy_data_field_rtoj(md->dfield[0], md->ypos_actual-md->yoff)+ 0.5)-1));    //if normal scan is done, show last complete profile
        pthread_mutex_unlock(md->dfield_mutex);
    }

    preview(md);

    /*return actual profile back to zero*/
    pthread_mutex_lock(md->dfield_mutex);
    md->actual_profile = md->actual_profile_candidate = 0;
    pthread_mutex_unlock(md->dfield_mutex);

    /*move back to offset values*/
    table_move_to(md, md->xoff, md->yoff);

    pthread_mutex_lock(md->scan_mutex);
    md->scanning = FALSE;
    pthread_mutex_unlock(md->scan_mutex);


    return NULL;
}

//scan command was already issued, get the data periodically and check the finish.
gpointer
scanloop_lua(gpointer data)
{    
    MiniData *md = (MiniData *)data;
    gint npos, ndata, lndata, n = 0, skip, report;
    gint preview_step = PREVIEW_STEP_POINTS;
    gboolean scanning;
    gchar buff[100];
    gint nprev;

    pthread_mutex_lock(md->scan_mutex);
    npos = md->npos;
    md->scan_start_time = md->time_actual;
    md->scan_end_time = 0;
    pthread_mutex_unlock(md->scan_mutex);

    get_actual_preview_step(md, &preview_step);

    writelog_date(md);
    g_snprintf(buff, sizeof(buff), "starting scanning through up to %d positions", npos);
    writelog(md, buff);

    skip = npos/20;
    report = npos/10;
   
    printf("starting scanning thread, npos = %d\n", npos); 

    set_scanning_script_parameter(md, "prase", 123);
    set_scanning_script_parameter(md, "sele", 124);
    set_scanning_script_parameter(md, "prase", 456);


    scanning = TRUE;
    n = nprev = lndata = ndata = 0;
    do {
        /*get data periodically and add them to result*/
        ndata = get_scan_ndata(md);

        if ((ndata-lndata)>50 || (md->npos-n)<50) {
            update_scan_path_data(md, md->vdata, &md->ndata, md->npos, lndata, ndata, n, 0, md->xypos);
            n += ndata-lndata;
            nprev += ndata-lndata;
            lndata = ndata;
        }
         
        //all completed, finish the scan, 
        if (n >= md->npos || get_scanning_script_status(md)==FALSE) scanning = FALSE;
 
        if (nprev > preview_step) //update at earliest moment after the preview nuber was reached
        {
            get_actual_preview_step(md, &preview_step);
            nprev = 0;
            preview(md);
        }

        /*update present profile index to show in graph and check if we were not stopped*/
        pthread_mutex_lock(md->dfield_mutex);
        md->actual_profile = md->actual_profile_candidate = 0; //no idea where to get this when we scan by script
        pthread_mutex_unlock(md->dfield_mutex);

        if (npos >= 1000)
        {
            if ((n%skip) == 0)
                save_backup(md);
            if ((n%report) == 0)
            {
                g_snprintf(buff, sizeof(buff), "%d%% positions completed (%d)", n / report * 10, npos);
                writelog(md, buff);
            }
        }
        usleep(100000);

    } while (n<npos && scanning==TRUE);

    if (npos < 1000)
    {
        save_backup(md);
        g_snprintf(buff, sizeof(buff), "%d%% positions completed (%d)", 100, npos);
        writelog(md, buff);
    }

    preview(md);

    /*return actual profile back to zero*/
    pthread_mutex_lock(md->dfield_mutex);
    md->actual_profile = md->actual_profile_candidate = 0;
    pthread_mutex_unlock(md->dfield_mutex);

    pthread_mutex_lock(md->scan_mutex);
    md->scanning = FALSE;
    pthread_mutex_unlock(md->scan_mutex);


    return NULL;
}



static void
prepare_datafield(GwyDataField *dfield, gint xres, gint yres, gdouble xreal, gdouble yreal)
{
    gwy_data_field_resample(dfield, xres, yres, GWY_INTERPOLATION_NONE);
    gwy_data_field_set_xreal(dfield, xreal);
    gwy_data_field_set_yreal(dfield, yreal);
    gwy_data_field_fill(dfield, 0);
}

//check if scanning in another thread was finished already and if yes, update controls*/
static gboolean
scanupdate(gpointer data)
{
    gboolean scanning, creating_path;
    MiniData *md = (MiniData *)data;

    pthread_mutex_lock(md->scan_mutex);
    scanning = md->scanning;
    pthread_mutex_unlock(md->scan_mutex);

    pthread_mutex_lock(md->create_path_mutex);
    creating_path = md->creating_path;
    pthread_mutex_unlock(md->create_path_mutex);

    if (scanning || creating_path) 
        return TRUE;
    
    md->state = GWS_NOTHING;
    md->scan_end_time = md->time_actual;
    scan_widgets_activate(md, TRUE);
    gtk_button_set_label(GTK_BUTTON(md->button_scan), "Scan");
    gtk_widget_set_sensitive(md->button_scan, TRUE);
    gws_changraph_change_state(md->graph[0], CHGR_STATE_FREERUNNING);
    gws_changraph_change_state(md->graph[1], CHGR_STATE_FREERUNNING);
   
    writelog_date(md);
    writelog(md, "scanning finished");
 
    return FALSE;
}


gint get_path_length(const MiniData *md)
{
    gint npos = 0;

    if (md->pathval == PATH_REGULAR) {
        npos = gwyscan_create_path_regular(NULL, md->xres, md->yres,
                                           md->xreal, md->yreal,
                                           md->xoff, md->yoff,
                                           md->rotate, GWYSCAN_DIRECTION_FORWARD);
    }
    if (md->pathval == PATH_REFINE) {
        npos = gwyscan_create_path_regular(NULL, md->xres, md->yres,
                                           md->xreal, md->yreal,
                                           md->xoff, md->yoff,
                                           md->rotate, GWYSCAN_DIRECTION_FORWARD_REFINE);
    }
    if (md->pathval == PATH_ALTERNATEBLOCK) {
        npos = gwyscan_create_path_regular(NULL, md->xres, md->yres,
                                           md->xreal, md->yreal,
                                           md->xoff, md->yoff,
                                           md->rotate, GWYSCAN_DIRECTION_FORWARD_ALTERNATEBLOCK);
    }
    else if (md->pathval == PATH_SPACE_FILLING)
    {
        npos = gwyscan_create_path_space_filling(NULL, md->order,
                                                 md->xreal, md->yreal,
                                                 md->xoff, md->yoff);
    }
    else if (md->pathval == PATH_SPIRAL)
    {
        npos = gwyscan_create_path_spiral(NULL, md->nvals,
                                          md->xreal, md->yreal,
                                          md->xoff, md->yoff);
    }
    else if (md->pathval == PATH_RANDOM)
    {
        npos = gwyscan_create_path_random(NULL, md->nvals,
                                          md->xreal, md->yreal,
                                          md->xoff, md->yoff);
    }
    else if (md->pathval == PATH_OCTAVE_1D)
    {
        npos = gwyscan_create_path_octave_profiles(NULL, md->order, md->noctaves, md->nprofiles,
                                                   md->xreal, md->yreal,
                                                   md->xoff, md->yoff);
    }
    else if (md->pathval == PATH_OCTAVE_2D)
    {
        npos = gwyscan_create_path_octave_2d(NULL, md->order, md->noctaves,
                                             md->xreal, md->yreal,
                                             md->xoff, md->yoff);
    }
    else if (md->pathval == PATH_WICHMANN)
    {
        npos = gwyscan_create_path_wichmann_profiles(NULL, md->nvals, md->nprofiles,
                                                     md->xreal, md->yreal,
                                                     md->xoff, md->yoff);
    }

    return npos;
}

gpointer 
create_path_loop(gpointer data)
{
    printf("creating path started\n");
    MiniData *md = (MiniData *)data; 

//    file_clear_cb(NULL, md); //erase the path

    pthread_mutex_lock(md->create_path_mutex);
    md->creating_path = TRUE;    
    pthread_mutex_unlock(md->create_path_mutex);

    md->npos = get_path_length(md);

    md->xypos = (gdouble *)g_malloc(2 * md->npos*sizeof(gdouble));    

	if (md->pathval == PATH_REGULAR) {
	    md->npos = gwyscan_create_path_regular(md->xypos, md->xres, md->yres,
			                                   md->xreal, md->yreal,
					                           md->xoff, md->yoff,
					                           md->rotate, GWYSCAN_DIRECTION_FORWARD);
	}
	if (md->pathval == PATH_REFINE) {
	    md->npos = gwyscan_create_path_regular(md->xypos, md->xres, md->yres,
			                                   md->xreal, md->yreal,
					                           md->xoff, md->yoff,
					                           md->rotate, GWYSCAN_DIRECTION_FORWARD_REFINE);
	}
	if (md->pathval == PATH_ALTERNATEBLOCK) {
	    md->npos = gwyscan_create_path_regular(md->xypos, md->xres, md->yres,
			                                   md->xreal, md->yreal,
					                           md->xoff, md->yoff,
					                           md->rotate, GWYSCAN_DIRECTION_FORWARD_ALTERNATEBLOCK);
	}
	else if (md->pathval == PATH_SPACE_FILLING)
	{
            md->npos = gwyscan_create_path_space_filling(md->xypos, md->order,
			                                             md->xreal, md->yreal,
					                                     md->xoff, md->yoff);
	}
	else if (md->pathval == PATH_SPIRAL)
	{
            md->npos = gwyscan_create_path_spiral(md->xypos, md->nvals,
			                                      md->xreal, md->yreal,
					                              md->xoff, md->yoff);
	}
	else if (md->pathval == PATH_RANDOM)
	{
            md->npos = gwyscan_create_path_random(md->xypos, md->nvals,
			                                      md->xreal, md->yreal,
					                              md->xoff, md->yoff);
	}
	else if (md->pathval == PATH_OCTAVE_1D)
	{
            md->npos = gwyscan_create_path_octave_profiles(md->xypos, md->order, md->noctaves, md->nprofiles,
			                                               md->xreal, md->yreal,
					                                       md->xoff, md->yoff);
	}
	else if (md->pathval == PATH_OCTAVE_2D)
	{
            md->npos = gwyscan_create_path_octave_2d(md->xypos, md->order, md->noctaves,
			                                         md->xreal, md->yreal,
					                                 md->xoff, md->yoff);
	}
	else if (md->pathval == PATH_WICHMANN)
	{
            md->npos = gwyscan_create_path_wichmann_profiles(md->xypos, md->nvals, md->nprofiles,
			                                                 md->xreal, md->yreal,
					                                         md->xoff, md->yoff);
	}

    printf("creating path finished\n");    

    pthread_mutex_lock(md->create_path_mutex);
    md->creating_path = FALSE;    
    pthread_mutex_unlock(md->create_path_mutex);

    writelog(md, "creating scan path finished");

    return NULL;
}

void
create_path(MiniData *md)
{
    file_clear_cb(NULL, md); //erase the path

    /*create scan path*/
    md->create_path_loop = g_thread_new("miniscan_create_path_thread", create_path_loop, md);

    writelog(md, "starting creating scan path");

    printf("create path thread started\n");
}

/*main scanning function: prepare path, run scanning thread, run completition waiting thread*/
void scan(MiniData *md, gboolean is_preview)
{
    gint i;
    gboolean creating_path = FALSE;
    gboolean adaptive, lua, isramp;
    gdouble xreal, yreal;
    gchar *buffer;
    GtkTextIter start;
    GtkTextIter end;

    if (md->scanning) {
        fprintf(stderr, "Error: scan already started\n");
        return; 
    }

    if (md->page == 1) adaptive = TRUE;
    else adaptive = FALSE;

    if (md->page == 7) lua = TRUE;
    else lua = FALSE;

    isramp = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(md->ramp_do_check));
    if (isramp) adaptive = FALSE; //we don't know how to handle it now

    md->ndata = 0;

    //start creating path
    if (adaptive) 
    {
       md->xypos = NULL;
       create_path(md);
       
       //wait until creating scan path is completed
       do {
           usleep(1000000);
           pthread_mutex_lock(md->create_path_mutex);
           creating_path = md->creating_path;
           pthread_mutex_unlock(md->create_path_mutex);
       } while (creating_path == TRUE);
    } 
    else if (lua)
    {
       md->npos = md->lua_size; //maximum allocation on server will be used here as well
       md->xypos = (gdouble *)g_malloc(2 * md->npos*sizeof(gdouble));
    }
    else 
    {
       //we have to create a fake path as well
       md->npos = gwyscan_create_path_regular(NULL, md->rxres, md->ryres,
                                           md->rxreal, md->ryreal,
                                           md->rxoff, md->ryoff,
                                           md->rrotate, GWYSCAN_DIRECTION_FORWARD);

       md->xypos = (gdouble *)g_malloc(2 * md->npos*sizeof(gdouble));
       gwyscan_create_path_regular(md->xypos, md->rxres, md->ryres,
                                           md->rxreal, md->ryreal,
                                           md->rxoff, md->ryoff,
                                           md->rrotate, GWYSCAN_DIRECTION_FORWARD);

       if (md->c_sc_reverse) { //create fake path even here
           md->rnpos = gwyscan_create_path_regular(NULL, md->rxres, md->ryres,
                                           md->rxreal, md->ryreal,
                                           md->rxoff, md->ryoff,
                                           md->rrotate, GWYSCAN_DIRECTION_REVERSE); 

          md->rxypos = (gdouble *)g_malloc(2 * md->rnpos*sizeof(gdouble));
          gwyscan_create_path_regular(md->rxypos, md->rxres, md->ryres,
                                           md->rxreal, md->ryreal,
                                           md->rxoff, md->ryoff,
                                           md->rrotate, GWYSCAN_DIRECTION_REVERSE);
     }
    }
    //allocate raw data for whole set of points 
    if (md->vdata != NULL) {
        g_free(md->vdata);
    }

    md->vdata = (gdouble *)g_malloc(md->nchan*md->npos*sizeof(gdouble)); 
    memset(md->vdata, 0, md->nchan*md->npos * sizeof(gdouble));
    g_assert(md->vdata != NULL);

    if (md->c_sc_reverse) {
       md->rvdata = (gdouble *)g_malloc(md->nchan*md->rnpos*sizeof(gdouble));
       memset(md->rvdata, 0, md->nchan*md->rnpos * sizeof(gdouble));
       g_assert(md->rvdata != NULL);
    }

    pthread_mutex_lock(md->scan_mutex);
    md->scanning = TRUE;
    pthread_mutex_unlock(md->scan_mutex);

    printf("set scan storage\n");    

    //set the channels that should be stored and sent from server.
    set_scan_storage(md, md->c_sc_a1, md->c_sc_p1, md->c_sc_in, FALSE);

    printf("set scan storage done\n");

    /*hack to be able to directly fill the datafield*/
    if (adaptive) {
       xreal = md->xreal;
       yreal = md->yreal;
    } 
    else if (lua)
    {
       xreal = md->rxreal;  //this is wrong, but I don't know how to handle it now
       yreal = md->ryreal;
    }
    else {
       xreal = md->rxreal;
       yreal = md->ryreal;
    }

    for (i=0; i<md->nchan; i++) prepare_datafield(md->dfield[i], PREVIEW_SIZE_PIXELS, PREVIEW_SIZE_PIXELS, xreal, yreal);
    if (md->c_sc_reverse) for (i=0; i<md->nchan; i++) prepare_datafield(md->rdfield[i], PREVIEW_SIZE_PIXELS, PREVIEW_SIZE_PIXELS, xreal, yreal);

    printf("prepare detafield done\n");

    //run the whole adaptive scan, note that line scans are run in the scan_loop only
    if (isramp) printf("ramp mode\n");
    else if (adaptive) run_scan_adaptive(md);
    else if (lua) {

        gtk_widget_set_sensitive(md->luaview, FALSE);
        gtk_text_buffer_get_start_iter(md->luabuffer, &start);
        gtk_text_buffer_get_end_iter(md->luabuffer, &end);
        buffer = gtk_text_buffer_get_text(md->luabuffer, &start, &end, FALSE);
        gtk_widget_set_sensitive(md->luaview, TRUE);

        printf("run scan script\n");
        run_scan_script(md, buffer);
        printf("run scan script done\n");
        
    }


    /*start scanning*/
    printf("start scanning thread\n");
    if (isramp) md->scan_loop = g_thread_new("miniscan_scan", scanloop_ramp, md);
    else if (adaptive) md->scan_loop = g_thread_new("miniscan_scan", scanloop_adaptive, md);
    else if (lua) md->scan_loop = g_thread_new("miniscan_scan", scanloop_lua, md);
    else md->scan_loop = g_thread_new("miniscan_scan", scanloop_line, md);

    /*now system is scanning, perodic update the result in GUI is done automatically*/

    printf("scan thread started\n");
    /*wait until scanning is completed*/
    g_timeout_add(1000, scanupdate, md);
}

/*main scanning function: prepare path, run scanning thread, run completition waiting thread*/
void ramp(MiniData *md)
{
    gint n, ramp_nchan, ndata, lndata;

    //allocate raw data for whole set of points 
    if (md->rampvdata != NULL) {
        g_free(md->rampvdata);
    }

    md->ramp_ndata = 0;
    md->ramp_npos = 2*md->ramp_zres;
    ramp_nchan = md->nchan + 1; //there is the zeroth q value channel in addition  

    printf("allocating ramp data %d x %d  = %d\n", ramp_nchan, md->ramp_npos, ramp_nchan*md->ramp_npos);

    md->rampvdata = (gdouble *)g_malloc(ramp_nchan*md->ramp_npos*sizeof(gdouble));  
    memset(md->rampvdata, 0, ramp_nchan*md->ramp_npos * sizeof(gdouble));
    g_assert(md->rampvdata != NULL);
    
    //set the channels that should be stored and sent from server, FIXME, this should reflect some settings
    set_ramp_storage(md, md->c_sc_a1, md->c_sc_p1, md->c_sc_in, FALSE);

    //run a single ramp on server
    run_ramp(md, md->ramp_quantity, md->ramp_zres, md->ramp_zfrom, md->ramp_zto, md->ramp_starttime, md->ramp_peaktime, md->ramp_time, md->ramp_time);

    //get all the data
    n = ndata = lndata = 0;
    do {
        ndata = get_ramp_ndata(md);
       // printf("n = %d, ramp ndata = %d  lndata = %d\n", n, ndata, lndata);

        if ((ndata-lndata)>50 || ((md->ramp_npos-n)<50 && ndata-lndata>0)) {
            update_ramp_data(md, md->rampvdata, &md->ramp_ndata, md->ramp_npos, lndata, ndata, n, 0);
            n += ndata-lndata;
            lndata = ndata;
        }
        usleep(10000);
     }
     while (n<md->ramp_npos);
   
     //print the measured data
//     for (i=0; i<md->ramp_npos; i++) printf("%d %g %g %g\n", i, md->rampvdata[0*md->ramp_npos + i], md->rampvdata[3*md->ramp_npos + i], md->rampvdata[4*md->ramp_npos + i]);


     //visualisation is handled by changraph automatically
}

/*update position values on GUI*/
static gboolean
valsupdate(gpointer data)
{
    gchar buffer[400];
    gdouble capxpos, capypos, zpiezo, error, in1, in2, timestamp, a1, p1, in[16];
    gdouble hw_start_time, now_time, scan_start_time, scan_end_time, scan_elapsed_time, scan_remaining_time;
    int set_secs, set_mins, set_hours, srt_secs, srt_mins, srt_hours;
    MiniData *md = (MiniData *)data;    

    if (md->closingapp) 
        return FALSE;

    get_actual_pos(md, &capxpos, &capypos);
    get_actual_zdata(md, &zpiezo, &error, &in1, &in2, &timestamp, &a1, &p1);
    get_actual_hw_start_time(md, &hw_start_time);
    get_actual_in(md, in);
    get_actual_scan_times(md, &scan_start_time, &scan_end_time);
    
    g_snprintf(buffer, sizeof(buffer), "%+3.3f", error);
    gtk_label_set_text(GTK_LABEL(md->error_label), buffer);    
    g_snprintf(buffer, sizeof(buffer), "%+3.3f", zpiezo*1e6);
    gtk_label_set_text(GTK_LABEL(md->zpiezo_label), buffer);    

    if (md->mode == GWS_MODE_AKIYAMA)
    {
       g_snprintf(buffer, sizeof(buffer), "%+3.3f", p1);
       gtk_label_set_text(GTK_LABEL(md->fmerror_label), buffer);    
    }

    // scan time elapsed    
    set_secs = set_mins = set_hours = srt_secs = srt_mins = srt_hours = 0;
    now_time = timestamp;// +hw_start_time;
    scan_elapsed_time = (md->scanning == TRUE)?now_time - scan_start_time:scan_end_time - scan_start_time;

    set_secs = ((int)scan_elapsed_time % 60);
    set_mins = (int)(((int)scan_elapsed_time / 60) % 60);
    set_hours = (int)((int)scan_elapsed_time) / 3600;

    // scan time estimated        
    if (md->scanning == TRUE && md->ndata > 0)
    {
        scan_remaining_time = ((gdouble)(md->npos - md->ndata) / md->ndata) * scan_elapsed_time;

        srt_secs = ((int)scan_remaining_time % 60);
        srt_mins = (int)(((int)scan_remaining_time / 60) % 60);
        srt_hours = (int)((int)scan_remaining_time / 3600);
    }

    // hwserver time elapsed
    int hwst_secs = ((int)timestamp % 60);
    int hwst_mins = (int)(((int)timestamp % 3600) / 60);
    int hwst_hours = (int)(((int)timestamp % 86400) / 3600);
    int hwst_days = (int)(timestamp / (60 * 60 * 24));


    g_snprintf(buffer, sizeof(buffer), "Table: <tt>(%8.5f, %8.5f)</tt> μm,  Z piezo: <tt>%+8.5f</tt> μm,  Error signal: <tt>%+8.5f</tt> V, IN1: <tt>%+8.5f</tt> IN2: <tt>%+8.5f</tt>  Server elapsed: <tt>%02d</tt>::<tt>%02d</tt>:<tt>%02d</tt>:<tt>%02d</tt> s",
               capxpos*1e6, capypos*1e6, zpiezo*1e6, error, in[0], in[1], hwst_days, hwst_hours, hwst_mins, hwst_secs);

    gwy_statusbar_set_markup(GWY_STATUSBAR(md->statusbar_measure), buffer);

    g_snprintf(buffer, sizeof(buffer), "Scan elapsed: <tt>%02d</tt>:<tt>%02d</tt>:<tt>%02d</tt> s,  Scan remaining: <tt>%02d</tt>:<tt>%02d</tt>:<tt>%02d</tt> s,  Measured: <tt>%6d</tt> points",
               set_hours, set_mins, set_secs, srt_hours, srt_mins, srt_secs, md->ndata);

    gwy_statusbar_set_markup(GWY_STATUSBAR(md->statusbar_scan), buffer);

    return TRUE;
}


static void 
scan_cb(GtkWidget *widget, MiniData *md)
{
    if (md->state == GWS_SCANNING)
    {
        printf("scan stopped\n");
        md->state = GWS_NOTHING;
        stop_scan(md);
        md->scanning = FALSE;   //force loop to exit
        scan_widgets_activate(md, TRUE);
	gtk_button_set_label(GTK_BUTTON(widget), "Scan");
        gws_changraph_change_state(md->graph[0], CHGR_STATE_FREERUNNING);
	gws_changraph_change_state(md->graph[1], CHGR_STATE_FREERUNNING);

    } else {
        printf("scan started\n");
        md->state = GWS_SCANNING;
        scan_widgets_activate(md, FALSE);
	gtk_button_set_label(GTK_BUTTON(widget), "Stop");
	gws_changraph_change_state(md->graph[0], CHGR_STATE_PROFILES);
	gws_changraph_change_state(md->graph[1], CHGR_STATE_PROFILES);

        scan_widgets_read(md);
        scan(md, FALSE);

    }
}

static void 
pause_cb(GtkWidget *widget, MiniData *md)
{
    if (md->pause == 1) {
         printf("scan unpaused\n");
         pause_scan(md, 0);
         md->pause = 0;
    }
    else {
         printf("scan paused\n");
         pause_scan(md, 1);
         md->pause = 1;
    }
}

static void 
createscanpath_cb(GtkWidget *widget, MiniData *md)
{    
    create_path(md);

}


static void
threshold_changed_cb(GtkObject *object,
                  MiniData* md)
{
   md->threshold = gtk_adjustment_get_value(GTK_ADJUSTMENT(object));
   set_setpoint(md, md->threshold);
}

static void
wait_changed_cb(GtkObject *object, MiniData* md)
{
    md->wait = gtk_adjustment_get_value(GTK_ADJUSTMENT(object));
    set_wait(md, md->wait);
}

static void
speed_changed_cb(GtkObject *object, MiniData* md)
{
    md->speed = gtk_adjustment_get_value(GTK_ADJUSTMENT(object));
    set_speed(md, md->speed);
}

static void
pidp_changed_cb(GtkObject *object, MiniData* md)
{
    md->pidp = gtk_adjustment_get_value(GTK_ADJUSTMENT(object));
    set_pid(md, md->pidp, md->pidi, md->pidd);
}
static void
pidi_changed_cb(GtkObject *object, MiniData* md)
{
    md->pidi = gtk_adjustment_get_value(GTK_ADJUSTMENT(object));
    set_pid(md, md->pidp, md->pidi, md->pidd);
}
static void
pidd_changed_cb(GtkObject *object, MiniData* md)
{
    md->pidd = gtk_adjustment_get_value(GTK_ADJUSTMENT(object));
    set_pid(md, md->pidp, md->pidi, md->pidd);
}
static void
fmpidp_changed_cb(GtkObject *object, MiniData* md)
{
    md->fmpidp = gtk_adjustment_get_value(GTK_ADJUSTMENT(object));
    set_fmpid(md, md->fmpidp, md->fmpidi, md->fmpidd);
}
static void
fmpidi_changed_cb(GtkObject *object, MiniData* md)
{
    md->fmpidi = gtk_adjustment_get_value(GTK_ADJUSTMENT(object));
    set_fmpid(md, md->fmpidp, md->fmpidi, md->fmpidd);
}
static void
fmpidd_changed_cb(GtkObject *object, MiniData* md)
{
    md->fmpidd = gtk_adjustment_get_value(GTK_ADJUSTMENT(object));
    set_fmpid(md, md->fmpidp, md->fmpidi, md->fmpidd);
}
static void
fmsetpoint_changed_cb(GtkObject *object, MiniData* md)
{
    md->fmsetpoint = gtk_adjustment_get_value(GTK_ADJUSTMENT(object));
    set_fmsetpoint(md, md->fmsetpoint);
}
static void
zpiezo_changed_cb(GtkObject *object, MiniData* md)
{
    md->zpiezo = gtk_adjustment_get_value(GTK_ADJUSTMENT(object));
    set_zpiezo(md, md->zpiezo*1e-6);
}

void quit_app_cb(MiniData *md)
{
   printf("closing application\n");
   writelog(md, "closing application...");
   md->closingapp = TRUE;

   usleep(2000000);

   gtk_main_quit();
}

void spectrum_cb(GtkWidget *menuitem, MiniData *md)
{
   GtkWidget *fv;  
   GtkWidget *dialog; 
   gdouble f, buf, a, p;

   dialog = gtk_dialog_new_with_buttons(_("Spectral probe response"),
                                         NULL, 0,
                                         GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                         GTK_STOCK_OK, GTK_RESPONSE_OK,
                                         NULL);
   gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_OK);

   set_feedback(md, FALSE);
   set_fmfeedback(md, FALSE);

   fv = gws_freqview_new((gpointer)md);
   gws_freqview_set_inits(GWS_FREQVIEW(fv), md->freq_from, md->freq_to, md->freq_step, md->exc1, md->exc2, md->freq_delay, md->f, md->freq_aout0, md->phaseshift1);

   gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), fv, TRUE, TRUE, 0);
   
   if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_OK)
   {
      f = gws_freqview_get_freq(GWS_FREQVIEW(fv)); 
      set_freq(md, 1, f*1000, GWS_FREQVIEW(fv)->exc1, 0); //from kHz to Hz
      set_freq(md, 2, f*1000, GWS_FREQVIEW(fv)->exc2, 0); //from kHz to Hz
      md->f = f;
      md->exc1 = GWS_FREQVIEW(fv)->exc1;
      md->exc2 = GWS_FREQVIEW(fv)->exc2;
      printf("Frequency set to %g kHz\n", f);

      md->freq_from = GWS_FREQVIEW(fv)->from;
      md->freq_to = GWS_FREQVIEW(fv)->to;
      md->freq_step = GWS_FREQVIEW(fv)->step;
      md->freq_delay = GWS_FREQVIEW(fv)->delay;
      md->freq_aout0 = GWS_FREQVIEW(fv)->aout1;
      md->phaseshift1 = GWS_FREQVIEW(fv)->shift;

      //in akiyama mode also set the FM feedback on, with setpoint as phase in selected frequency
      if (md->mode == GWS_MODE_AKIYAMA) {
         set_fmpid(md, md->fmpidp, md->fmpidi, md->fmpidd);
         usleep(100000);
         get_actual_zdata(md, &buf, &buf, &buf, &buf, &buf, &a, &p);
         printf("actual amplitude %g  phase %g, phase will be set as FM setpoint\n", a, p);
         gtk_adjustment_set_value(GTK_ADJUSTMENT(md->fmsetpoint_adj), p);
         set_fmsetpoint(md, p);
         printf("fm setpoint value %g\n", gtk_adjustment_get_value(GTK_ADJUSTMENT(md->fmsetpoint_adj)));
         usleep(100000);
         set_fmfeedback(md, 1);
      }


   }
   gtk_widget_destroy(dialog);

   //set_feedback(md, TRUE);

}

void laser_cb(GtkWidget *menuitem, MiniData *md)
{
   GtkWidget *fv;  
   GtkWidget *dialog; 
   GwsLaserView *laserview;

   dialog = gtk_dialog_new_with_buttons(_("Optical deflection response"),
                                         NULL, 0,
                                         GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                         GTK_STOCK_OK, GTK_RESPONSE_OK,
                                         NULL);
   gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_OK);


   fv = gws_laserview_new((gpointer)md);
   laserview = (GwsLaserView *) fv;
   gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), fv, TRUE, TRUE, 0);
   
   if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_OK)
   {
   }
   laserview->closing = 1;
   usleep(200000);
   gtk_widget_destroy(dialog);

}
void table_cb(GtkWidget *menuitem, MiniData *md)
{
   GtkWidget *fv;  
   GtkWidget *dialog;
   gdouble xpos, ypos; 
   GwsTableView *tableview;

   dialog = gtk_dialog_new_with_buttons(_("Table manual movement"),
                                         NULL, 0,
                                         GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                         GTK_STOCK_OK, GTK_RESPONSE_OK,
                                         NULL);
   gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_OK);


   fv = gws_tableview_new((gpointer)md);
   tableview = (GwsTableView *) fv;
   get_actual_pos(md, &xpos, &ypos);
   gws_tableview_set_values(tableview, md->xrange, md->yrange, xpos, ypos, md->speed);
 
   gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), fv, TRUE, TRUE, 0);
   
   if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_OK)
   {
      gtk_adjustment_set_value(GTK_ADJUSTMENT(md->speed_adj), md->speed);
   }
   tableview->closing = 1;
   usleep(200000);
   gtk_widget_destroy(dialog);

}

void signals_cb(GtkWidget *menuitem, MiniData *md)
{
   GtkWidget *fv;  
   GtkWidget *dialog;
   GwsSignals *signals;
   gint oldmode;

   dialog = gtk_dialog_new_with_buttons(_("Signals settings and readout"),
                                         NULL, 0,
                                         GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                         GTK_STOCK_OK, GTK_RESPONSE_OK,
                                         NULL);
   gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_OK);

   oldmode = md->mode;

   fv = gws_signals_new((gpointer)md);
   signals = (GwsSignals *) fv;
 
   gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), fv, TRUE, TRUE, 0);
   
   if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_OK)
   {
   }
   signals->closing = 1;
  
   md->mode = oldmode;
   set_mode(md, md->mode);
 
   usleep(200000);
   gtk_widget_destroy(dialog);

}

void update_preview_channels(MiniData *md)
{
   gws_chanview_set_running(md->view[0], FALSE);
   gws_chanview_set_running(md->view[1], FALSE);
   gws_chanview_set_running(md->view[2], FALSE);
   gws_chanview_set_running(md->view[3], FALSE);
   gws_changraph_set_running(md->graph[0], FALSE);
   gws_changraph_set_running(md->graph[1], FALSE);

   usleep(100000);

   update_preview_channels_data(md);

   gws_chanview_refresh_channels(md->view[0]);
   gws_chanview_refresh_channels(md->view[1]);
   gws_chanview_refresh_channels(md->view[2]);
   gws_chanview_refresh_channels(md->view[3]);
   gws_changraph_refresh_channels(md->graph[0]);
   gws_changraph_refresh_channels(md->graph[1]);

   gws_chanview_enable_reverse(md->view[0], md->c_sc_reverse);
   gws_chanview_enable_reverse(md->view[1], md->c_sc_reverse);
   gws_chanview_enable_reverse(md->view[2], md->c_sc_reverse);
   gws_chanview_enable_reverse(md->view[3], md->c_sc_reverse);

   gws_chanview_set_running(md->view[0], TRUE);
   gws_chanview_set_running(md->view[1], TRUE);
   gws_chanview_set_running(md->view[2], TRUE);
   gws_chanview_set_running(md->view[3], TRUE);
   gws_changraph_set_running(md->graph[0], TRUE);
   gws_changraph_set_running(md->graph[1], TRUE);


}

void channels_cb(GtkWidget *menuitem, MiniData *md)
{
   GtkWidget *ch;  
   GtkWidget *dialog; 
   gint i;

   dialog = gtk_dialog_new_with_buttons(_("Scan storage settings"),
                                         NULL, 0,
                                         GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                         GTK_STOCK_OK, GTK_RESPONSE_OK,
                                         NULL);
   gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_OK);

   ch = gws_channels_new((gpointer)md);
   gws_channels_set_inits(GWS_CHANNELS(ch), md->c_sc_in, md->c_sc_reverse);

   gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), ch, TRUE, TRUE, 0);
   
   if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_OK)
   {
      md->c_sc_reverse = GWS_CHANNELS(ch)->reverse;
      for (i=0; i<16; i++) md->c_sc_in[i] = GWS_CHANNELS(ch)->in[i];
   }
   gtk_widget_destroy(dialog);

   update_preview_channels(md);
   set_standby_storage(md, md->c_sc_a1, md->c_sc_p1, md->c_sc_in, FALSE);

}
 

void path_changed_cb(GtkComboBox *combo, MiniData *md)
{
   md->pathval = gtk_combo_box_get_active(combo); 

   if (md->pathval==PATH_REGULAR || md->pathval==PATH_ALTERNATEBLOCK || md->pathval==PATH_REFINE) 
   {
      printf("Regular path chosen\n");
      gtk_widget_show(md->xres_spin);
      gtk_widget_show(md->xres_label);

      gtk_widget_show(md->yres_spin);
      gtk_widget_show(md->yres_label);

      gtk_widget_show(md->rotate_spin);
      gtk_widget_show(md->rotate_label);

      gtk_widget_show(md->profile_check);
      gtk_widget_show(md->profile_spin);

      gtk_widget_hide(md->nvals_spin);
      gtk_widget_hide(md->nvals_label);
      
      gtk_widget_hide(md->order_spin);
      gtk_widget_hide(md->order_label);

      gtk_widget_hide(md->nprofiles_spin);
      gtk_widget_hide(md->nprofiles_label);

      gtk_widget_hide(md->noctaves_spin);
      gtk_widget_hide(md->noctaves_label);
 
   } 
   else if (md->pathval==PATH_SPACE_FILLING) 
   {
      printf("Space filling path chosen\n");
      gtk_widget_hide(md->xres_spin);
      gtk_widget_hide(md->xres_label);

      gtk_widget_hide(md->yres_spin);
      gtk_widget_hide(md->yres_label);

      gtk_widget_hide(md->rotate_spin);
      gtk_widget_hide(md->rotate_label);

      gtk_widget_hide(md->profile_check);
      gtk_widget_hide(md->profile_spin);

      gtk_widget_hide(md->nvals_spin);
      gtk_widget_hide(md->nvals_label);
      
      gtk_widget_show(md->order_spin);
      gtk_widget_show(md->order_label);

      gtk_widget_hide(md->nprofiles_spin);
      gtk_widget_hide(md->nprofiles_label);

      gtk_widget_hide(md->noctaves_spin);
      gtk_widget_hide(md->noctaves_label);
 
   }
   else if (md->pathval==PATH_SPIRAL) 
   {
      printf("Spiral path chosen\n");
      gtk_widget_hide(md->xres_spin);
      gtk_widget_hide(md->xres_label);

      gtk_widget_hide(md->yres_spin);
      gtk_widget_hide(md->yres_label);

      gtk_widget_hide(md->rotate_spin);
      gtk_widget_hide(md->rotate_label);

      gtk_widget_hide(md->profile_check);
      gtk_widget_hide(md->profile_spin);

      gtk_widget_show(md->nvals_spin);
      gtk_widget_show(md->nvals_label);
      
      gtk_widget_hide(md->order_spin);
      gtk_widget_hide(md->order_label);

      gtk_widget_hide(md->nprofiles_spin);
      gtk_widget_hide(md->nprofiles_label);

      gtk_widget_hide(md->noctaves_spin);
      gtk_widget_hide(md->noctaves_label);
 
   }
   else if (md->pathval==PATH_RANDOM) 
   {
      printf("Random path chosen\n");
      gtk_widget_hide(md->xres_spin);
      gtk_widget_hide(md->xres_label);

      gtk_widget_hide(md->yres_spin);
      gtk_widget_hide(md->yres_label);

      gtk_widget_hide(md->rotate_spin);
      gtk_widget_hide(md->rotate_label);

      gtk_widget_hide(md->profile_check);
      gtk_widget_hide(md->profile_spin);

      gtk_widget_show(md->nvals_spin);
      gtk_widget_show(md->nvals_label);
      
      gtk_widget_hide(md->order_spin);
      gtk_widget_hide(md->order_label);

      gtk_widget_hide(md->nprofiles_spin);
      gtk_widget_hide(md->nprofiles_label);

      gtk_widget_hide(md->noctaves_spin);
      gtk_widget_hide(md->noctaves_label);
 
   }
   else if (md->pathval==PATH_OCTAVE_1D) 
   {
      printf("Octave 1D path chosen\n");
      gtk_widget_hide(md->xres_spin);
      gtk_widget_hide(md->xres_label);

      gtk_widget_hide(md->yres_spin);
      gtk_widget_hide(md->yres_label);

      gtk_widget_hide(md->rotate_spin);
      gtk_widget_hide(md->rotate_label);

      gtk_widget_hide(md->profile_check);
      gtk_widget_hide(md->profile_spin);

      gtk_widget_hide(md->nvals_spin);
      gtk_widget_hide(md->nvals_label);
      
      gtk_widget_show(md->order_spin);
      gtk_widget_show(md->order_label);

      gtk_widget_show(md->nprofiles_spin);
      gtk_widget_show(md->nprofiles_label);

      gtk_widget_show(md->noctaves_spin);
      gtk_widget_show(md->noctaves_label);
 
   }
   else if (md->pathval==PATH_OCTAVE_2D) 
   {
      printf("Octave 2D path chosen\n");
      gtk_widget_hide(md->xres_spin);
      gtk_widget_hide(md->xres_label);

      gtk_widget_hide(md->yres_spin);
      gtk_widget_hide(md->yres_label);

      gtk_widget_hide(md->rotate_spin);
      gtk_widget_hide(md->rotate_label);

      gtk_widget_hide(md->profile_check);
      gtk_widget_hide(md->profile_spin);

      gtk_widget_hide(md->nvals_spin);
      gtk_widget_hide(md->nvals_label);
      
      gtk_widget_show(md->order_spin);
      gtk_widget_show(md->order_label);

      gtk_widget_hide(md->nprofiles_spin);
      gtk_widget_hide(md->nprofiles_label);

      gtk_widget_show(md->noctaves_spin);
      gtk_widget_show(md->noctaves_label);
 
   }
   else if (md->pathval==PATH_WICHMANN) 
   {
      printf("Wichmann path chosen\n");
      gtk_widget_hide(md->xres_spin);
      gtk_widget_hide(md->xres_label);

      gtk_widget_hide(md->yres_spin);
      gtk_widget_hide(md->yres_label);

      gtk_widget_hide(md->rotate_spin);
      gtk_widget_hide(md->rotate_label);

      gtk_widget_hide(md->profile_check);
      gtk_widget_hide(md->profile_spin);

      gtk_widget_show(md->nvals_spin);
      gtk_widget_show(md->nvals_label);
      
      gtk_widget_hide(md->order_spin);
      gtk_widget_hide(md->order_label);

      gtk_widget_show(md->nprofiles_spin);
      gtk_widget_show(md->nprofiles_label);

      gtk_widget_hide(md->noctaves_spin);
      gtk_widget_hide(md->noctaves_label);
 
   }

   //re-generate path
   scan_widgets_read(md);
    

//   create_path(md);
}

static void
show_hide_widgets(MiniData *md)
{
    if (md->mode == GWS_MODE_AKIYAMA) {
        gtk_widget_show(md->fmpid_label);
        gtk_widget_show(md->fmpidp_spin);
        gtk_widget_show(md->fmpidi_spin);
        gtk_widget_show(md->fmpidd_spin);
        gtk_widget_show(md->fmerror_label);
        gtk_widget_show(md->fmsetpoint_label);
        gtk_widget_show(md->fmsetpoint_spin);
        gtk_widget_show(md->fm_label);
        gtk_widget_show(md->fmunit_label);
        gtk_widget_show(md->fmunitc_label);
        gtk_widget_show(md->phasefilter_check);
        gtk_widget_show(md->amfm_check);
        gtk_widget_show(md->fmlimit_combo);
        gtk_widget_show(md->phaselimit_combo);
        
    } else {
        gtk_widget_hide(md->fmpid_label);
        gtk_widget_hide(md->fmpidp_spin);
        gtk_widget_hide(md->fmpidi_spin);
        gtk_widget_hide(md->fmpidd_spin);
        gtk_widget_hide(md->fmerror_label);
        gtk_widget_hide(md->fmsetpoint_label);
        gtk_widget_hide(md->fmsetpoint_spin);
        gtk_widget_hide(md->fm_label);
        gtk_widget_hide(md->fmunit_label);
        gtk_widget_hide(md->fmunitc_label);
        gtk_widget_hide(md->phasefilter_check);
        gtk_widget_hide(md->amfm_check);
        gtk_widget_hide(md->fmlimit_combo);
        gtk_widget_hide(md->phaselimit_combo);
     }
}

static void
mode_changed(GtkComboBox *combo, MiniData *md)
{
    gchar buffer[100];

    md->mode = gwy_enum_combo_box_get_active(combo);
    printf("mode changed\n");

    //setpoint units
    if (md->fbunitc_label != NULL && md->fbunit_label != NULL)
    {
       if (md->mode == GWS_MODE_OFF || md->mode == GWS_MODE_PROPORTIONAL || md->mode == GWS_MODE_NCAMPLITUDE || md->mode == GWS_MODE_STM || md->mode == GWS_MODE_NENOPROBE) {
          g_snprintf(buffer, sizeof(buffer), "V");
          gtk_label_set_text(GTK_LABEL(md->fbunit_label), buffer);
          g_snprintf(buffer, sizeof(buffer), "V,");
          gtk_label_set_text(GTK_LABEL(md->fbunitc_label), buffer);
       } else if (md->mode == GWS_MODE_NCPHASE) {
          g_snprintf(buffer, sizeof(buffer), "rad");
          gtk_label_set_text(GTK_LABEL(md->fbunit_label), buffer);
          g_snprintf(buffer, sizeof(buffer), "rad,");
          gtk_label_set_text(GTK_LABEL(md->fbunitc_label), buffer);
       } else if (md->mode == GWS_MODE_AKIYAMA) {
          g_snprintf(buffer, sizeof(buffer), "Hz");
          gtk_label_set_text(GTK_LABEL(md->fbunit_label), buffer);
          g_snprintf(buffer, sizeof(buffer), "Hz,");
          gtk_label_set_text(GTK_LABEL(md->fbunitc_label), buffer);
       }
    } else printf("labels are null!\n");

    //adjust button
    if (md->mode == GWS_MODE_OFF || md->mode == GWS_MODE_STM || md->mode == GWS_MODE_NENOPROBE)    
    {
        gtk_widget_set_sensitive(md->button_adjust, FALSE);
    } 
    else if (md->mode == GWS_MODE_PROPORTIONAL || md->mode == GWS_MODE_NCAMPLITUDE || md->mode == GWS_MODE_NCPHASE || md->mode == GWS_MODE_AKIYAMA)
    {
        gtk_widget_set_sensitive(md->button_adjust, TRUE);
    }

    //approach/retract button
    if (md->mode == GWS_MODE_OFF) 
    { 
       gtk_widget_set_sensitive(md->button_approach, FALSE);
       gtk_widget_set_sensitive(md->button_retract, FALSE);
    } else
    {
       gtk_widget_set_sensitive(md->button_approach, TRUE);
       gtk_widget_set_sensitive(md->button_retract, TRUE);
    }

    //show or hide whole widgets
    show_hide_widgets(md);

    //adjust scan channels
    if (md->mode == GWS_MODE_OFF || md->mode == GWS_MODE_PROPORTIONAL || md->mode == GWS_MODE_STM) 
    {
        md->c_sc_a1 = TRUE; //was FALSE, now we want to oscillate in contact mode as well
        md->c_sc_a2 = FALSE;
        md->c_sc_p1 = TRUE; //was FALSE
        md->c_sc_p2 = FALSE;
    } else {
        md->c_sc_a1 = TRUE;
        md->c_sc_a2 = FALSE;
        md->c_sc_p1 = TRUE;
        md->c_sc_p2 = FALSE;
    }
    update_preview_channels(md);

}

static void 
adjust_cb(GtkWidget *widget, MiniData *md)
{
    if (md->mode == GWS_MODE_NCAMPLITUDE || md->mode == GWS_MODE_NCPHASE || md->mode == GWS_MODE_AKIYAMA)
    {
        spectrum_cb(NULL, md);
    } 
    else if (md->mode == GWS_MODE_PROPORTIONAL) 
    {
        laser_cb(NULL, md);
    }
}

static void 
ramp_cb(GtkWidget *widget, MiniData *md)
{
    printf("ramp!\n");

    scan_widgets_read(md);
    ramp(md);

}

static void 
lua_load_cb(GtkWidget *widget, MiniData *md)
{
    gchar *buffer, *filename;
    GError *err = NULL;
    gint result;

    GtkWidget *dialog;

    dialog = gtk_file_chooser_dialog_new ("Load Lua script...",
                                      GTK_WINDOW(md->toplevel),
                                      GTK_FILE_CHOOSER_ACTION_OPEN,
                                      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                      GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
                                      NULL);

    if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
    {
        filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));

        if (filename != NULL)        
        {
           result = g_file_get_contents(filename, &buffer, NULL, &err);
           if (result == FALSE)
           {
              g_error_free(err);
           }
           else { 
              gtk_widget_set_sensitive(md->luaview, FALSE);
              gtk_text_buffer_set_text(md->luabuffer, buffer, -1);       
              gtk_widget_set_sensitive(md->luaview, TRUE);
              g_free(buffer); 
           }
           g_free(filename);
        }

    }
    gtk_widget_destroy(dialog);
}

static void 
lua_save_cb(GtkWidget *widget, MiniData *md)
{
    gchar *buffer, *filename;
    GtkTextIter start;
    GtkTextIter end;
    GError *err = NULL;
    gint result;

    GtkWidget *dialog;

    dialog = gtk_file_chooser_dialog_new ("Save Lua script as...",
                                      GTK_WINDOW(md->toplevel),
                                      GTK_FILE_CHOOSER_ACTION_SAVE,
                                      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                      GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
                                      NULL);

    gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (dialog), TRUE);

    if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
    {
        filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));

        gtk_widget_set_sensitive(md->luaview, FALSE);
        gtk_text_buffer_get_start_iter(md->luabuffer, &start);
        gtk_text_buffer_get_end_iter(md->luabuffer, &end);
        buffer = gtk_text_buffer_get_text(md->luabuffer, &start, &end, FALSE);       
        gtk_widget_set_sensitive(md->luaview, TRUE);

        /* set the contents of the file to the text from the buffer */
        if (filename != NULL)        
        {
           result = g_file_set_contents(filename, buffer, -1, &err);

           if (result == FALSE)
           {
              g_error_free(err);
           }
           g_free(filename);
        }

        g_free(buffer); 
    }
    gtk_widget_destroy(dialog);
}

static void
rampmode_changed_cb(GtkComboBox *combo, MiniData *md)
{
    gchar buffer[100];

    md->ramp_quantity = md->rampmode = gwy_enum_combo_box_get_active(combo);

    //setpoint units
    if (md->rampqc_label != NULL && md->rampq_label != NULL)
    {
       if (md->rampmode == GWS_RAMP_ZPIEZO) {
          g_snprintf(buffer, sizeof(buffer), "μm,");
          gtk_label_set_text(GTK_LABEL(md->rampqc_label), buffer);
          g_snprintf(buffer, sizeof(buffer), "μm");
          gtk_label_set_text(GTK_LABEL(md->rampq_label), buffer);
       } else {
          g_snprintf(buffer, sizeof(buffer), "mV,");
          gtk_label_set_text(GTK_LABEL(md->rampqc_label), buffer);
          g_snprintf(buffer, sizeof(buffer), "mV");
          gtk_label_set_text(GTK_LABEL(md->rampq_label), buffer);
       }
    } else printf("labels are null!\n");
}
static void
liftmode_changed_cb(GtkComboBox *combo, MiniData *md)
{
    md->liftmode = gwy_enum_combo_box_get_active(combo);

    if (md->liftmode == GWS_LIFT_FEEDBACK)
    {
        gtk_widget_set_sensitive(md->lift1_height_spin, FALSE);
        gtk_widget_set_sensitive(md->lift2_height_spin, FALSE);
        gtk_widget_set_sensitive(md->lift3_height_spin, FALSE);
        gtk_widget_set_sensitive(md->lift4_height_spin, FALSE);
        gtk_widget_set_sensitive(md->lift5_height_spin, FALSE);
    }
    else {
        if (md->nlift>0) gtk_widget_set_sensitive(md->lift1_height_spin, TRUE);
        if (md->nlift>1) gtk_widget_set_sensitive(md->lift2_height_spin, TRUE);
        if (md->nlift>2) gtk_widget_set_sensitive(md->lift3_height_spin, TRUE);
        if (md->nlift>3) gtk_widget_set_sensitive(md->lift4_height_spin, TRUE);
        if (md->nlift>4) gtk_widget_set_sensitive(md->lift5_height_spin, TRUE);
    }

}
static void
liftswitch_changed_cb(GtkComboBox *combo, MiniData *md)
{
    md->lift_switch = gwy_enum_combo_box_get_active(combo);

    if (md->lift_switch==GWS_LIFTSWITCH_NOTHING) {
        gtk_widget_set_sensitive(md->lift1_value_spin, FALSE);
        gtk_widget_set_sensitive(md->lift2_value_spin, FALSE);
        gtk_widget_set_sensitive(md->lift3_value_spin, FALSE);
        gtk_widget_set_sensitive(md->lift4_value_spin, FALSE);
        gtk_widget_set_sensitive(md->lift5_value_spin, FALSE);
    } 
    else {
        if (md->nlift>0) gtk_widget_set_sensitive(md->lift1_value_spin, TRUE);
        if (md->nlift>1) gtk_widget_set_sensitive(md->lift2_value_spin, TRUE);
        if (md->nlift>2) gtk_widget_set_sensitive(md->lift3_value_spin, TRUE);
        if (md->nlift>3) gtk_widget_set_sensitive(md->lift4_value_spin, TRUE);
        if (md->nlift>4) gtk_widget_set_sensitive(md->lift5_value_spin, TRUE);
    }
}

static void
nlift_changed_cb(GtkObject *object, MiniData* md)
{
    md->nlift = gtk_adjustment_get_value(GTK_ADJUSTMENT(object));

    if (md->in_setup) return; 
 
    gtk_widget_set_sensitive(md->lift1_height_spin, FALSE);
    gtk_widget_set_sensitive(md->lift1_value_spin, FALSE);
    gtk_widget_set_sensitive(md->lift2_height_spin, FALSE);
    gtk_widget_set_sensitive(md->lift2_value_spin, FALSE);
    gtk_widget_set_sensitive(md->lift3_height_spin, FALSE);
    gtk_widget_set_sensitive(md->lift3_value_spin, FALSE);
    gtk_widget_set_sensitive(md->lift4_height_spin, FALSE);
    gtk_widget_set_sensitive(md->lift4_value_spin, FALSE);
    gtk_widget_set_sensitive(md->lift5_height_spin, FALSE);
    gtk_widget_set_sensitive(md->lift5_value_spin, FALSE);

    if (md->nlift>0) {
       if (md->liftmode != GWS_LIFT_FEEDBACK) gtk_widget_set_sensitive(md->lift1_height_spin, TRUE);
       if (md->lift_switch != GWS_LIFTSWITCH_NOTHING) gtk_widget_set_sensitive(md->lift1_value_spin, TRUE);
    }
    if (md->nlift>1) {
       if (md->liftmode != GWS_LIFT_FEEDBACK) gtk_widget_set_sensitive(md->lift2_height_spin, TRUE);
       if (md->lift_switch != GWS_LIFTSWITCH_NOTHING) gtk_widget_set_sensitive(md->lift2_value_spin, TRUE);
    }
    if (md->nlift>2) {
       if (md->liftmode != GWS_LIFT_FEEDBACK) gtk_widget_set_sensitive(md->lift3_height_spin, TRUE);
       if (md->lift_switch != GWS_LIFTSWITCH_NOTHING) gtk_widget_set_sensitive(md->lift3_value_spin, TRUE);
    }
    if (md->nlift>3) {
       if (md->liftmode != GWS_LIFT_FEEDBACK) gtk_widget_set_sensitive(md->lift4_height_spin, TRUE);
       if (md->lift_switch != GWS_LIFTSWITCH_NOTHING) gtk_widget_set_sensitive(md->lift4_value_spin, TRUE);
    }
    if (md->nlift>4) {
       if (md->liftmode != GWS_LIFT_FEEDBACK) gtk_widget_set_sensitive(md->lift5_height_spin, TRUE);
       if (md->lift_switch != GWS_LIFTSWITCH_NOTHING) gtk_widget_set_sensitive(md->lift5_value_spin, TRUE);
    }

    update_preview_channels(md);

}

static void
liftvalues_changed_cb(GtkObject *object, MiniData* md)
{
    md->lift1_height = gtk_adjustment_get_value(GTK_ADJUSTMENT(md->lift1_height_adj))*1e-6;
    md->lift2_height = gtk_adjustment_get_value(GTK_ADJUSTMENT(md->lift2_height_adj))*1e-6;
    md->lift3_height = gtk_adjustment_get_value(GTK_ADJUSTMENT(md->lift3_height_adj))*1e-6;
    md->lift4_height = gtk_adjustment_get_value(GTK_ADJUSTMENT(md->lift4_height_adj))*1e-6;
    md->lift5_height = gtk_adjustment_get_value(GTK_ADJUSTMENT(md->lift5_height_adj))*1e-6;

    md->lift1_value = gtk_adjustment_get_value(GTK_ADJUSTMENT(md->lift1_value_adj));
    md->lift2_value = gtk_adjustment_get_value(GTK_ADJUSTMENT(md->lift2_value_adj));
    md->lift3_value = gtk_adjustment_get_value(GTK_ADJUSTMENT(md->lift3_value_adj));
    md->lift4_value = gtk_adjustment_get_value(GTK_ADJUSTMENT(md->lift4_value_adj));
    md->lift5_value = gtk_adjustment_get_value(GTK_ADJUSTMENT(md->lift5_value_adj));
}
 
static void
out1rule_changed_cb(GtkComboBox *combo, MiniData *md)
{
    md->out1rule = gwy_enum_combo_box_get_active(combo);

    if (md->out1rule == GWS_RULE_NOTHING) gtk_widget_set_sensitive(md->out1value_spin, TRUE);
    else gtk_widget_set_sensitive(md->out1value_spin, FALSE);

    set_routing(md, "out1", md->out1rule); //FIXME, note that routing does not set the fixed value, but GUI seems as it would do it

}
static void
out2rule_changed_cb(GtkComboBox *combo, MiniData *md)
{
    md->out2rule = gwy_enum_combo_box_get_active(combo);

    if (md->out2rule == GWS_RULE_NOTHING) gtk_widget_set_sensitive(md->out2value_spin, TRUE);
    else gtk_widget_set_sensitive(md->out2value_spin, FALSE);

    set_routing(md, "out2", md->out2rule);  //FIXME, note that routing does not set the fixed value, but GUI seems as it would do it
}

static void
phaselimit_changed_cb(GtkComboBox *combo, MiniData *md)
{
    md->fm_phase_limit = gwy_enum_combo_box_get_active(combo);

    set_phaselimit(md, md->fm_phase_limit); 
}
static void
fmlimit_changed_cb(GtkComboBox *combo, MiniData *md)
{
    md->fm_frequency_limit = gwy_enum_combo_box_get_active(combo);

    set_fmlimit(md, md->fm_frequency_limit);
}

static void
phasefilter_toggled_cb(GtkToggleButton *togglebutton, MiniData *md)
{
    md->phasefilter = gtk_toggle_button_get_active(togglebutton);
    
    set_filterphase(md, md->phasefilter);
}
static void
amfm_toggled_cb(GtkToggleButton *togglebutton, MiniData *md)
{
    md->amfm = gtk_toggle_button_get_active(togglebutton);
    
    set_fmam_feedback(md, md->amfm);
}

static void
kpfm_routing_changed_cb(GtkComboBox *combo, MiniData *md)
{
    md->kpfm_routing = gwy_enum_combo_box_get_active(combo);

    if (md->kpfm_routing == GWS_ROUTE_NOWHERE)
    {
        gtk_widget_set_sensitive(md->kpfm_frequency_spin, FALSE);
        gtk_widget_set_sensitive(md->kpfm_amplitude_spin, FALSE);
        gtk_widget_set_sensitive(md->kpfm_offset_spin, FALSE);
        gtk_widget_set_sensitive(md->kpfm_phase_spin, FALSE);
        gtk_widget_set_sensitive(md->kpfm_bw_spin, FALSE);
    } else {
        gtk_widget_set_sensitive(md->kpfm_frequency_spin, TRUE);
        gtk_widget_set_sensitive(md->kpfm_amplitude_spin, TRUE);
        gtk_widget_set_sensitive(md->kpfm_offset_spin, TRUE);
        gtk_widget_set_sensitive(md->kpfm_phase_spin, TRUE);
        gtk_widget_set_sensitive(md->kpfm_bw_spin, TRUE);
    }
}

static void
litho_routing_changed_cb(GtkComboBox *combo, MiniData *md)
{
    md->litho_routing = gwy_enum_combo_box_get_active(combo);
}

static void 
litho_load_cb(GtkWidget *widget, MiniData *md)
{
    gchar *buffer, *filename;
    GError *err = NULL;
    gint result;

    GtkWidget *dialog;

    dialog = gtk_file_chooser_dialog_new ("Load litography script...",
                                      GTK_WINDOW(md->toplevel),
                                      GTK_FILE_CHOOSER_ACTION_OPEN,
                                      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                      GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
                                      NULL);

    if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
    {
        filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));

        if (filename != NULL)        
        {
           result = g_file_get_contents(filename, &buffer, NULL, &err);
           if (result == FALSE)
           {
              g_error_free(err);
           }
           else { 
              gtk_widget_set_sensitive(md->lithoview, FALSE);
              gtk_text_buffer_set_text(md->lithobuffer, buffer, -1);       
              gtk_widget_set_sensitive(md->lithoview, TRUE);
              g_free(buffer); 
           }
           g_free(filename);
        }

    }
    gtk_widget_destroy(dialog);
}

static void 
litho_save_cb(GtkWidget *widget, MiniData *md)
{
    gchar *buffer, *filename;
    GtkTextIter start;
    GtkTextIter end;
    GError *err = NULL;
    gint result;

    GtkWidget *dialog;

    dialog = gtk_file_chooser_dialog_new ("Save litography script as...",
                                      GTK_WINDOW(md->toplevel),
                                      GTK_FILE_CHOOSER_ACTION_SAVE,
                                      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                      GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
                                      NULL);

    gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (dialog), TRUE);

    if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
    {
        filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));

        gtk_widget_set_sensitive(md->lithoview, FALSE);
        gtk_text_buffer_get_start_iter(md->lithobuffer, &start);
        gtk_text_buffer_get_end_iter(md->lithobuffer, &end);
        buffer = gtk_text_buffer_get_text(md->lithobuffer, &start, &end, FALSE);       
        gtk_widget_set_sensitive(md->lithoview, TRUE);

        /* set the contents of the file to the text from the buffer */
        if (filename != NULL)        
        {
           result = g_file_set_contents(filename, buffer, -1, &err);

           if (result == FALSE)
           {
              g_error_free(err);
           }
           g_free(filename);
        }

        g_free(buffer); 
    }
    gtk_widget_destroy(dialog);
}

static void 
litho_run_cb(GtkWidget *widget, MiniData *md)
{
    printf("litho run!\n");
}


void
init_hw(MiniData *md)
{
    printf("sending the GUI settings to hwserver\n");
    set_wait(md, md->wait);
    set_speed(md, md->speed);
    set_pid(md, md->pidp, md->pidi, md->pidd);
    set_fmpid(md, md->fmpidp, md->fmpidi, md->fmpidd);
    set_fmsetpoint(md, md->fmsetpoint);
    set_filter(md, 0, md->filter1);
    set_filter(md, 1, md->filter2);
}

void 
set_ranges(MiniData *md)
{
    gtk_adjustment_set_upper(GTK_ADJUSTMENT(md->xreal_adj), md->xrange/1e-6);
    gtk_adjustment_set_upper(GTK_ADJUSTMENT(md->yreal_adj), md->yrange/1e-6);
    gtk_adjustment_set_upper(GTK_ADJUSTMENT(md->rxreal_adj), md->xrange/1e-6);
    gtk_adjustment_set_upper(GTK_ADJUSTMENT(md->ryreal_adj), md->yrange/1e-6);

    if (gtk_adjustment_get_value(GTK_ADJUSTMENT(md->xreal_adj)) > md->xrange/1e-6) 
          gtk_adjustment_set_value(GTK_ADJUSTMENT(md->xreal_adj), md->xrange/1e-6);

    if (gtk_adjustment_get_value(GTK_ADJUSTMENT(md->yreal_adj)) > md->yrange/1e-6) 
          gtk_adjustment_set_value(GTK_ADJUSTMENT(md->yreal_adj), md->yrange/1e-6);

    if (gtk_adjustment_get_value(GTK_ADJUSTMENT(md->rxreal_adj)) > md->xrange/1e-6) 
          gtk_adjustment_set_value(GTK_ADJUSTMENT(md->rxreal_adj), md->xrange/1e-6);

    if (gtk_adjustment_get_value(GTK_ADJUSTMENT(md->ryreal_adj)) > md->yrange/1e-6) 
          gtk_adjustment_set_value(GTK_ADJUSTMENT(md->ryreal_adj), md->yrange/1e-6);

}


GtkWidget*
create_window(MiniData *md)
{
   GtkWidget *window, *label, *hbox, *hbox2, *vbox, *vboxleft, *vboxright, *frame, *notebook;
   GtkWidget *menubar, *page, *scroll, *table, *spin, *vbox2;
   GtkWidget *file, *filemenu, *menuitem, *align, *instrument, *instmenu;
   gint row;

    static const GwyEnum modes[] = {
        { N_("Off"),                         GWS_MODE_OFF,          },
        { N_("Optical, contact mode"),       GWS_MODE_PROPORTIONAL, },
        { N_("Optical, tapping amplitude"),  GWS_MODE_NCAMPLITUDE,  },
        { N_("Optical, tapping phase"),      GWS_MODE_NCPHASE,      },
        { N_("Akiyama probe"),               GWS_MODE_AKIYAMA,      },
        { N_("Nenoprobe"),                   GWS_MODE_NENOPROBE,    },
        { N_("STM probe"),                   GWS_MODE_STM,          },
    };

   static const GwyEnum ramps[] = {
        { N_("Z piezo"),                     GWS_RAMP_ZPIEZO,       },
        { N_("Tip bias"),                    GWS_RAMP_TIPV,         },
        { N_("Sample bias"),                 GWS_RAMP_SAMPLEV,      },
        { N_("OUT 1"),                       GWS_RAMP_OUT1,         },
        { N_("OUT 2"),                       GWS_RAMP_OUT2,         },
    };

   static const GwyEnum lifts[] = {
        { N_("Keep feedback"),               GWS_LIFT_FEEDBACK,      },
        { N_("Profile"),                     GWS_LIFT_PROFILE,       },
        { N_("Profile leveled"),             GWS_LIFT_LPROFILE,      },
        { N_("Image"),                       GWS_LIFT_IMAGE,         },
        { N_("Image leveled"),               GWS_LIFT_LIMAGE,        },
        { N_("Point"),                       GWS_LIFT_POINT,         },
    };

   static const GwyEnum switches[] = {
        { N_("Nothing"),                     GWS_LIFTSWITCH_NOTHING,      },
        { N_("Tip bias"),                    GWS_LIFTSWITCH_TIPV,         },
        { N_("Sample bias"),                 GWS_LIFTSWITCH_SAMPLEV,      },
        { N_("Out 1"),                       GWS_LIFTSWITCH_OUT1,         },
        { N_("Out 2"),                       GWS_LIFTSWITCH_OUT2,         },
    };

   static const GwyEnum outs[] = {
        { N_("Nothing"),                     GWS_RULE_NOTHING,   },
        { N_("X position"),                  GWS_RULE_X,         },
        { N_("Y position"),                  GWS_RULE_Y,         },
        { N_("Z piezo"),                     GWS_RULE_Z,         },
        { N_("In 1"),                        GWS_RULE_IN1,       },
        { N_("In 2"),                        GWS_RULE_IN2,       },
    };

   static const GwyEnum routs[] = {
        { N_("Nowhere"),                     GWS_ROUTE_NOWHERE,     },
        { N_("Tip"),                         GWS_ROUTE_TIP,     },
        { N_("Sample"),                      GWS_ROUTE_SAMPLE,  },
        { N_("Out 1"),                       GWS_ROUTE_OUT1,  },
        { N_("Out 2"),                       GWS_ROUTE_OUT2,  },
    };

   static const GwyEnum fmlimits[] = {
        { N_("max ±3.75 Hz"),              GWS_FMLIMIT_0,  },
        { N_("max ±7.5 Hz"),               GWS_FMLIMIT_1,  },
        { N_("max ±15 Hz"),                GWS_FMLIMIT_2   },
        { N_("max ±30 Hz"),                GWS_FMLIMIT_3,  },
        { N_("max ±60 Hz"),                GWS_FMLIMIT_4,  },
        { N_("max ±120 Hz"),               GWS_FMLIMIT_5,  },
        { N_("max ±240 Hz"),               GWS_FMLIMIT_6,  },
        { N_("max ±480 Hz"),               GWS_FMLIMIT_7,  },
    };

   static const GwyEnum phaselimits[] = {
        { N_("max ±4 deg"),                 GWS_PHASELIMIT_0,  },
        { N_("max ±8 deg"),                 GWS_PHASELIMIT_1,  },
        { N_("max ±16 deg"),                GWS_PHASELIMIT_2,  },
        { N_("max ±32 deg"),                GWS_PHASELIMIT_3,  },
        { N_("max ±64 deg"),                GWS_PHASELIMIT_4,  },
        { N_("max ±128 deg"),               GWS_PHASELIMIT_5,  },
        { N_("unlimited"),                  GWS_PHASELIMIT_6,  },
    };

    md->toplevel = window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    gtk_window_set_title(GTK_WINDOW(window), MINISCAN_APP_NAME);
    gtk_widget_set_size_request(window, 1340, 1000);

    g_signal_connect_swapped(G_OBJECT(window), "destroy", 
        G_CALLBACK(quit_app_cb), md);


    vbox = gtk_vbox_new(FALSE, 0);
    gtk_container_add(GTK_CONTAINER(window), vbox);

    /*Menu bar*/
    menubar = gtk_menu_bar_new();
    filemenu = gtk_menu_new();
    instmenu = gtk_menu_new();

    file = gtk_menu_item_new_with_mnemonic("_File");
    gtk_menu_item_set_submenu(GTK_MENU_ITEM(file), filemenu);

    menuitem = gtk_menu_item_new_with_mnemonic("_Clear");
    gtk_menu_shell_append(GTK_MENU_SHELL(filemenu), menuitem);
//    g_signal_connect(G_OBJECT(menuitem), "activate",
//        G_CALLBACK(file_clear_cb), md);
    gtk_widget_set_tooltip_text(GTK_WIDGET(menuitem), "Clear measured points.");

    menuitem = gtk_menu_item_new_with_mnemonic("_Save XYZ");
    gtk_menu_shell_append(GTK_MENU_SHELL(filemenu), menuitem);
    g_signal_connect(G_OBJECT(menuitem), "activate",
        G_CALLBACK(file_save_xyz_cb), md);
    gtk_widget_set_tooltip_text(GTK_WIDGET(menuitem), "Save data (*.gxyzf)");

   
    menuitem = gtk_menu_item_new_with_mnemonic("Save XYZ _as");
    gtk_menu_shell_append(GTK_MENU_SHELL(filemenu), menuitem);
    g_signal_connect(G_OBJECT(menuitem), "activate",
        G_CALLBACK(file_save_xyz_as_cb), md);
    gtk_widget_set_tooltip_text(GTK_WIDGET(menuitem), "Save XYZ data as (*.gxyzf)");

    menuitem = gtk_menu_item_new_with_mnemonic("Save _brick data as");
    gtk_menu_shell_append(GTK_MENU_SHELL(filemenu), menuitem);
    g_signal_connect(G_OBJECT(menuitem), "activate",
        G_CALLBACK(file_save_brick_as_cb), md);
    gtk_widget_set_tooltip_text(GTK_WIDGET(menuitem), "Save brick data as");

    menuitem = gtk_menu_item_new_with_mnemonic("Save _preview");
    gtk_menu_shell_append(GTK_MENU_SHELL(filemenu), menuitem);
    g_signal_connect(G_OBJECT(menuitem), "activate",
        G_CALLBACK(file_save_cb), md);
    gtk_widget_set_tooltip_text(GTK_WIDGET(menuitem), "Save preview data (*.gwy)");
   
    menuitem = gtk_menu_item_new_with_mnemonic("Save pre_view as");
    gtk_menu_shell_append(GTK_MENU_SHELL(filemenu), menuitem);
    g_signal_connect(G_OBJECT(menuitem), "activate",
        G_CALLBACK(file_save_as_cb), md);
    gtk_widget_set_tooltip_text(GTK_WIDGET(menuitem), "Save preview data as (*.gwy)");

    gtk_menu_shell_append(GTK_MENU_SHELL(menubar), file);
    gtk_box_pack_start(GTK_BOX(vbox), menubar, FALSE, FALSE, 3); 

    menuitem = gtk_separator_menu_item_new();
    gtk_menu_shell_append(GTK_MENU_SHELL(filemenu), menuitem);

    menuitem = gtk_menu_item_new_with_mnemonic("_Clear project");
    gtk_menu_shell_append(GTK_MENU_SHELL(filemenu), menuitem);
    g_signal_connect(G_OBJECT(menuitem), "activate",
        G_CALLBACK(project_clear_cb), md);
    gtk_widget_set_tooltip_text(GTK_WIDGET(menuitem), "Start new project");

    menuitem = gtk_menu_item_new_with_mnemonic("_Add data to project");
    gtk_menu_shell_append(GTK_MENU_SHELL(filemenu), menuitem);
    g_signal_connect(G_OBJECT(menuitem), "activate",
        G_CALLBACK(project_add_cb), md);
    gtk_widget_set_tooltip_text(GTK_WIDGET(menuitem), "Add current data to project");

    menuitem = gtk_menu_item_new_with_mnemonic("_Save project");
    gtk_menu_shell_append(GTK_MENU_SHELL(filemenu), menuitem);
    g_signal_connect(G_OBJECT(menuitem), "activate",
        G_CALLBACK(project_save_cb), md);
    gtk_widget_set_tooltip_text(GTK_WIDGET(menuitem), "Save project (*.gwy)");

    menuitem = gtk_menu_item_new_with_mnemonic("_Save project as...");
    gtk_menu_shell_append(GTK_MENU_SHELL(filemenu), menuitem);
    g_signal_connect(G_OBJECT(menuitem), "activate",
        G_CALLBACK(project_save_as_cb), md);
    gtk_widget_set_tooltip_text(GTK_WIDGET(menuitem), "Save project (*.gwy)");


    menuitem = gtk_separator_menu_item_new();
    gtk_menu_shell_append(GTK_MENU_SHELL(filemenu), menuitem);


    menuitem = gtk_menu_item_new_with_mnemonic("_Quit");
    gtk_menu_shell_append(GTK_MENU_SHELL(filemenu), menuitem);
    g_signal_connect_swapped(G_OBJECT(menuitem), "activate", G_CALLBACK(quit_app_cb), md);
    gtk_widget_set_tooltip_text(GTK_WIDGET(menuitem), "Quit this program");


    instrument = gtk_menu_item_new_with_mnemonic("_Instrument");
    gtk_menu_item_set_submenu(GTK_MENU_ITEM(instrument), instmenu);

    menuitem = gtk_menu_item_new_with_mnemonic("Scan _channels...");
    gtk_menu_shell_append(GTK_MENU_SHELL(instmenu), menuitem);
    g_signal_connect(G_OBJECT(menuitem), "activate",
        G_CALLBACK(channels_cb), md);
    gtk_widget_set_tooltip_text(GTK_WIDGET(menuitem), "Set channels to be acquired during scan");

    menuitem = gtk_menu_item_new_with_mnemonic("_Optical deflection setup...");
    gtk_menu_shell_append(GTK_MENU_SHELL(instmenu), menuitem);
    g_signal_connect(G_OBJECT(menuitem), "activate",
        G_CALLBACK(laser_cb), md);
    gtk_widget_set_tooltip_text(GTK_WIDGET(menuitem), "Align laser on the quadrant detector");

    menuitem = gtk_menu_item_new_with_mnemonic("_Non-contact setup...");
    gtk_menu_shell_append(GTK_MENU_SHELL(instmenu), menuitem);
    g_signal_connect(G_OBJECT(menuitem), "activate",
        G_CALLBACK(spectrum_cb), md);
    gtk_widget_set_tooltip_text(GTK_WIDGET(menuitem), "Get spectral response of the probe");

    menuitem = gtk_menu_item_new_with_mnemonic("_Move table...");
    gtk_menu_shell_append(GTK_MENU_SHELL(instmenu), menuitem);
    g_signal_connect(G_OBJECT(menuitem), "activate",
        G_CALLBACK(table_cb), md);
    gtk_widget_set_tooltip_text(GTK_WIDGET(menuitem), "Move table manually");

    menuitem = gtk_menu_item_new_with_mnemonic("_Reset hardware time");
    gtk_menu_shell_append(GTK_MENU_SHELL(instmenu), menuitem);
    g_signal_connect(G_OBJECT(menuitem), "activate",
        G_CALLBACK(hw_reset_time_cb), md);
    gtk_widget_set_tooltip_text(GTK_WIDGET(menuitem), "Reset time used for timestamps");

    menuitem = gtk_menu_item_new_with_mnemonic("_Get hwserver version");
    gtk_menu_shell_append(GTK_MENU_SHELL(instmenu), menuitem);
    g_signal_connect(G_OBJECT(menuitem), "activate",
        G_CALLBACK(hw_get_version_cb), md);
    gtk_widget_set_tooltip_text(GTK_WIDGET(menuitem), "Get version of hwserver we are talking to");

    menuitem = gtk_menu_item_new_with_mnemonic("Read and set raw _signals");
    gtk_menu_shell_append(GTK_MENU_SHELL(instmenu), menuitem);
    g_signal_connect(G_OBJECT(menuitem), "activate",
        G_CALLBACK(signals_cb), md);
    gtk_widget_set_tooltip_text(GTK_WIDGET(menuitem), "Read values of available input signals and set them");


    gtk_menu_shell_append(GTK_MENU_SHELL(menubar), instrument);
 
    /****  controls and data widgets  *****/
    hbox = gtk_hbox_new(FALSE, 0);
    gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);


    vboxleft = gtk_vbox_new(FALSE, 0);
    gtk_box_pack_start(GTK_BOX(hbox), vboxleft, TRUE, TRUE, 0);

    vboxright = gtk_vbox_new(FALSE, 0);
    gtk_box_pack_start(GTK_BOX(hbox), vboxright, TRUE, TRUE, 0);
    
    /****  Hardware settings  ****/
    frame = gtk_frame_new("Feedback and sensor settings");
    gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_ETCHED_IN);
    gtk_box_pack_start(GTK_BOX(vboxleft), frame, FALSE, TRUE, 0);

    vbox2 = gtk_vbox_new(FALSE, 0);
    gtk_container_add(GTK_CONTAINER(frame), vbox2);


    table = gtk_table_new(12, 7, FALSE);
    gtk_box_pack_start(GTK_BOX(vbox2), table, FALSE, TRUE, 0);
    row = 0;

    label = gtk_label_new("Mode:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label,
                   0, 1, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    md->mode_combo
        = gwy_enum_combo_box_new(modes, G_N_ELEMENTS(modes),
                                 G_CALLBACK(mode_changed),
                                 md, md->mode, TRUE);

    gtk_table_attach(GTK_TABLE(table), md->mode_combo,
                   1, 3, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    md->button_adjust = gtk_button_new_with_label("Adjust sensor");
    g_signal_connect(md->button_adjust, "clicked",
                             G_CALLBACK(adjust_cb), md);
    gtk_table_attach(GTK_TABLE(table), md->button_adjust,
                   3, 6, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

 
    row++;


    row++;
    label = gtk_label_new("Setpoint:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label,
                   0, 1, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    md->threshold_adj = gtk_adjustment_new(md->threshold, -100, 100, 0.01, 0.1, 0);
    spin = gtk_spin_button_new(GTK_ADJUSTMENT(md->threshold_adj), 1, 2);
    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(spin), 3);
    g_signal_connect(md->threshold_adj, "value-changed",
                     G_CALLBACK(threshold_changed_cb), md);
    gtk_table_attach(GTK_TABLE(table), spin,
                   1, 2, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    md->fbunitc_label = gtk_label_new("ADC,");
    gtk_misc_set_alignment(GTK_MISC(md->fbunitc_label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), md->fbunitc_label,
                   2, 3, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);


    label = gtk_label_new("actual value:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label,
                   3, 4, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    md->error_label = gtk_label_new("unknown");
    gtk_misc_set_alignment(GTK_MISC(md->error_label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), md->error_label,
                   4, 5, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    md->fbunit_label = gtk_label_new("ADC");
    gtk_misc_set_alignment(GTK_MISC(md->fbunit_label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), md->fbunit_label,
                   5, 6, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);
    row++;

    label = gtk_label_new("PID:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label,
                   0, 1, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    md->pidp_adj = gtk_adjustment_new(md->pidp, 0, 1.0, 0.0001, 0.001, 0);
    spin = gtk_spin_button_new(GTK_ADJUSTMENT(md->pidp_adj), 1, 2);
    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(spin), 4);
    g_signal_connect(md->pidp_adj, "value-changed",
                     G_CALLBACK(pidp_changed_cb), md);
    gtk_table_attach(GTK_TABLE(table), spin,
                   1, 2, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    md->pidi_adj = gtk_adjustment_new(md->pidi, 0, 1.0, 0.0001, 0.001, 0);
    spin = gtk_spin_button_new(GTK_ADJUSTMENT(md->pidi_adj), 1, 2);
    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(spin), 4);
    g_signal_connect(md->pidi_adj, "value-changed",
                     G_CALLBACK(pidi_changed_cb), md);
    gtk_table_attach(GTK_TABLE(table), spin,
                   2, 3, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    md->pidd_adj = gtk_adjustment_new(md->pidd, 0, 1.0, 0.0001, 0.001, 0);
    spin = gtk_spin_button_new(GTK_ADJUSTMENT(md->pidd_adj), 1, 2);
    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(spin), 4);
    g_signal_connect(md->pidd_adj, "value-changed",
                     G_CALLBACK(pidd_changed_cb), md);
    gtk_table_attach(GTK_TABLE(table), spin,
                   3, 4, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    row++;

    md->fmpid_label = gtk_label_new("FM PID:");
    gtk_misc_set_alignment(GTK_MISC(md->fmpid_label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), md->fmpid_label,
                   0, 1, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    md->fmpidp_adj = gtk_adjustment_new(md->fmpidp, 0, 1, 0.0001, 0.001, 0);
    md->fmpidp_spin = gtk_spin_button_new(GTK_ADJUSTMENT(md->fmpidp_adj), 1, 2);
    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(md->fmpidp_spin), 4);
    g_signal_connect(md->fmpidp_adj, "value-changed",
                     G_CALLBACK(fmpidp_changed_cb), md);
    gtk_table_attach(GTK_TABLE(table), md->fmpidp_spin,
                   1, 2, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    md->fmpidi_adj = gtk_adjustment_new(md->fmpidi, 0, 1, 0.0001, 0.001, 0);
    md->fmpidi_spin = gtk_spin_button_new(GTK_ADJUSTMENT(md->fmpidi_adj), 1, 2);
    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(md->fmpidi_spin), 4);
    g_signal_connect(md->fmpidi_adj, "value-changed",
                     G_CALLBACK(fmpidi_changed_cb), md);
    gtk_table_attach(GTK_TABLE(table), md->fmpidi_spin,
                   2, 3, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    md->fmpidd_adj = gtk_adjustment_new(md->fmpidd, 0, 1, 0.0001, 0.001, 0);
    md->fmpidd_spin = gtk_spin_button_new(GTK_ADJUSTMENT(md->fmpidd_adj), 1, 2);
    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(md->fmpidd_spin), 4);
    g_signal_connect(md->fmpidd_adj, "value-changed",
                     G_CALLBACK(fmpidd_changed_cb), md);
    gtk_table_attach(GTK_TABLE(table), md->fmpidd_spin,
                   3, 4, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    row++;

    md->fmsetpoint_label = gtk_label_new("FM setpoint:");
    gtk_misc_set_alignment(GTK_MISC(md->fmsetpoint_label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), md->fmsetpoint_label,
                     0, 1, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    md->fmsetpoint_adj = gtk_adjustment_new(md->fmsetpoint, -10, 10, 0.01, 1, 0);
    md->fmsetpoint_spin = gtk_spin_button_new(GTK_ADJUSTMENT(md->fmsetpoint_adj), 1, 2);
    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(md->fmsetpoint_spin), 3);
    g_signal_connect(md->fmsetpoint_adj, "value-changed",
                     G_CALLBACK(fmsetpoint_changed_cb), md);
    gtk_table_attach(GTK_TABLE(table), md->fmsetpoint_spin,
                   1, 2, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    md->fmunitc_label = gtk_label_new("rad, ");
    gtk_misc_set_alignment(GTK_MISC(md->fmunitc_label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), md->fmunitc_label,
                   2, 3, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    md->fm_label = gtk_label_new("actual value:");
    gtk_misc_set_alignment(GTK_MISC(md->fm_label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), md->fm_label,
                   3, 4, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    md->fmerror_label = gtk_label_new("unknown");
    gtk_misc_set_alignment(GTK_MISC(md->fmerror_label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), md->fmerror_label,
                   4, 5, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    md->fmunit_label = gtk_label_new("rad");
    gtk_misc_set_alignment(GTK_MISC(md->fmunit_label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), md->fmunit_label,
                   5, 6, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    row++;

    md->fmlimit_combo
        = gwy_enum_combo_box_new(fmlimits, G_N_ELEMENTS(fmlimits),
                                 G_CALLBACK(fmlimit_changed_cb),
                                 md, md->fm_frequency_limit, TRUE);

    gtk_table_attach(GTK_TABLE(table), md->fmlimit_combo,
                   0, 1, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    md->phaselimit_combo
        = gwy_enum_combo_box_new(phaselimits, G_N_ELEMENTS(phaselimits),
                                 G_CALLBACK(phaselimit_changed_cb),
                                 md, md->fm_phase_limit, TRUE);

    gtk_table_attach(GTK_TABLE(table), md->phaselimit_combo,
                   1, 2, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    md->phasefilter_check = gtk_check_button_new_with_mnemonic("phase LP");
    gtk_table_attach(GTK_TABLE(table), md->phasefilter_check,
                     2, 3, row, row+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);

    g_signal_connect(md->phasefilter_check, "toggled",
                     G_CALLBACK(phasefilter_toggled_cb), md);

    md->amfm_check = gtk_check_button_new_with_mnemonic("fix ampl.");
    gtk_table_attach(GTK_TABLE(table), md->amfm_check,
                     3, 4, row, row+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);

    g_signal_connect(md->amfm_check, "toggled",
                     G_CALLBACK(amfm_toggled_cb), md);



    row++;

    label = gtk_label_new("Z manual:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label,
                   0, 1, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    md->zpiezo_adj = gtk_adjustment_new(md->zpiezo, 0, 10, 0.0001, 0.0001, 0);
    md->zpiezo_spin = gtk_spin_button_new(GTK_ADJUSTMENT(md->zpiezo_adj), 1, 2);
    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(md->zpiezo_spin), 4);
    g_signal_connect(md->zpiezo_adj, "value-changed",
                     G_CALLBACK(zpiezo_changed_cb), md);
    gtk_table_attach(GTK_TABLE(table), md->zpiezo_spin,
                   1, 2, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    label = gtk_label_new("μm,");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label,
                   2, 3, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    label = gtk_label_new("actual value:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label,
                   3, 4, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    md->zpiezo_label = gtk_label_new("unknown");
    gtk_misc_set_alignment(GTK_MISC(md->zpiezo_label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), md->zpiezo_label,
                   4, 5, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    label = gtk_label_new("μm");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label,
                   5, 6, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);
    row++;


    table = gtk_table_new(15, 5, FALSE);
    gtk_box_pack_start(GTK_BOX(vbox2), table, FALSE, FALSE, 5);
    row = 0;


    hbox2 = gtk_hbox_new(TRUE, 0);
    gtk_box_pack_start(GTK_BOX(vbox2), hbox2, FALSE, FALSE, 0);
 
    md->button_approach = gtk_button_new_with_label("Feedback on");
    gtk_box_pack_start(GTK_BOX(hbox2), md->button_approach, FALSE, TRUE, 0);
    g_signal_connect(md->button_approach, "clicked",
                             G_CALLBACK(approach_cb), md);
 
    md->button_retract = gtk_button_new_with_label("Feedback off");
    gtk_box_pack_start(GTK_BOX(hbox2), md->button_retract, FALSE, TRUE, 0);
    g_signal_connect(md->button_retract, "clicked",
                             G_CALLBACK(retract_cb), md);

     md->button_tipup = gtk_button_new_with_label("Move up");
    gtk_box_pack_start(GTK_BOX(hbox2), md->button_tipup, FALSE, TRUE, 0);
    g_signal_connect(md->button_tipup, "clicked",
                             G_CALLBACK(tipup_cb), md);

     md->button_tipdown = gtk_button_new_with_label("Move down");
    gtk_box_pack_start(GTK_BOX(hbox2), md->button_tipdown, FALSE, TRUE, 0);
    g_signal_connect(md->button_tipdown, "clicked",
                             G_CALLBACK(tipdown_cb), md);

    /****  scan settings  ****/
    frame = gtk_frame_new("Scan settings");
    gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_ETCHED_IN);
    gtk_box_pack_start(GTK_BOX(vboxleft), frame, FALSE, FALSE, 0); 

    notebook = gtk_notebook_new();
    gtk_container_add(GTK_CONTAINER(frame), notebook);
    g_signal_connect_swapped(notebook, "switch-page",
                                 G_CALLBACK(scan_page_switched_cb), md);


    /***  regular ***/
    page = gtk_vbox_new(FALSE, 0);
    gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
                               page,
                               gtk_label_new(("Raster scan")));


    vbox2 = gtk_vbox_new(FALSE, 0);
    gtk_container_add(GTK_CONTAINER(page), vbox2);


    table = gtk_table_new(15, 5, FALSE);
    gtk_box_pack_start(GTK_BOX(vbox2), table, FALSE, FALSE, 10);
    row = 0;


    gtk_widget_set_size_request(table, -1, 250);

    md->rxres_label = gtk_label_new("Pixel resolution:");
    gtk_misc_set_alignment(GTK_MISC(md->rxres_label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), md->rxres_label,
                   0, 1, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);

    md->rxres_adj = gtk_adjustment_new(md->rxres, 1, 100000, 1, 10, 0);
    md->rxres_spin = gtk_spin_button_new(GTK_ADJUSTMENT(md->rxres_adj), 1, 2);
    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(md->rxres_spin), 0);
    gtk_table_attach(GTK_TABLE(table), md->rxres_spin,
                   1, 2, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);


    md->ryres_label = gtk_label_new("x");
    gtk_misc_set_alignment(GTK_MISC(md->ryres_label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), md->ryres_label,
                   2, 3, row, row+1, 0, 0, 5, 5);

    md->ryres_adj = gtk_adjustment_new(md->ryres, 1, 100000, 1, 10, 0);
    md->ryres_spin = gtk_spin_button_new(GTK_ADJUSTMENT(md->ryres_adj), 1, 2);
    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(md->ryres_spin), 0);
    gtk_table_attach(GTK_TABLE(table), md->ryres_spin,
                   3, 4, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    row++;

    label = gtk_label_new("Physical range [μm]:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label,
                   0, 1, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);

    md->rxreal_adj = gtk_adjustment_new(md->rxreal*1e6, 1, md->xrange*1e6, 1, 10, 0);
    md->rxreal_spin = gtk_spin_button_new(GTK_ADJUSTMENT(md->rxreal_adj), 1, 2);
    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(md->rxreal_spin), 2);
    gtk_table_attach(GTK_TABLE(table), md->rxreal_spin,
                   1, 2, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    label = gtk_label_new("x");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label,
                   2, 3, row, row+1, 0, 0, 5, 5);

    md->ryreal_adj = gtk_adjustment_new(md->ryreal*1e6, 1, md->yrange*1e6, 1, 10, 0);
    md->ryreal_spin = gtk_spin_button_new(GTK_ADJUSTMENT(md->ryreal_adj), 1, 2);
    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(md->ryreal_spin), 2);
    gtk_table_attach(GTK_TABLE(table), md->ryreal_spin,
                   3, 4, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    row++;

   printf("offset %g %g\n", md->rxoff, md->ryoff);

    label = gtk_label_new("Offset [μm]:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label,
                   0, 1, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);

    md->rxoff_adj = gtk_adjustment_new(md->rxoff*1e6, 0, md->xrange*1e6, 1, 10, 0);
    md->rxoff_spin = gtk_spin_button_new(GTK_ADJUSTMENT(md->rxoff_adj), 1, 2);
    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(md->rxoff_spin), 2);
    gtk_table_attach(GTK_TABLE(table), md->rxoff_spin,
                   1, 2, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    label = gtk_label_new(";");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label,
                   2, 3, row, row+1, 0, 0, 5, 5);

    md->ryoff_adj = gtk_adjustment_new(md->ryoff*1e6, 0, md->yrange*1e6, 1, 10, 0);
    md->ryoff_spin = gtk_spin_button_new(GTK_ADJUSTMENT(md->ryoff_adj), 1, 2);
    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(md->ryoff_spin), 2);
    gtk_table_attach(GTK_TABLE(table), md->ryoff_spin,
                   3, 4, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    row++;

    
    md->rrotate_label = gtk_label_new("Rotate whole scan area [deg]:");
    gtk_misc_set_alignment(GTK_MISC(md->rrotate_label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), md->rrotate_label,
                   0, 3, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);

    md->rrotate_adj = gtk_adjustment_new(md->rrotate, -180, 180, 1, 10, 0);
    md->rrotate_spin = gtk_spin_button_new(GTK_ADJUSTMENT(md->rrotate_adj), 1, 2);
    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(md->rrotate_spin), 1);
    gtk_table_attach(GTK_TABLE(table), md->rrotate_spin,
                   3, 4, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);
    row++;
    
    md->rprofile_check = gtk_check_button_new_with_mnemonic("repeat only single profile No.");
    gtk_table_attach(GTK_TABLE(table), md->rprofile_check,
                     0, 3, row, row+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);

    md->rprofile_adj = gtk_adjustment_new(md->rprofile, 0, 100000, 1, 10, 0);
    md->rprofile_spin = gtk_spin_button_new(GTK_ADJUSTMENT(md->rprofile_adj), 1, 2);
    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(md->rprofile_spin), 0);
    gtk_table_attach(GTK_TABLE(table), md->rprofile_spin,
                   3, 4, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);
    row++;


 
    /***  adaptive ***/
    page = gtk_vbox_new(FALSE, 0);
    gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
                               page,
                               gtk_label_new(("Path scan")));


    vbox2 = gtk_vbox_new(FALSE, 0);
    gtk_container_add(GTK_CONTAINER(page), vbox2);


    table = gtk_table_new(15, 5, FALSE);
    gtk_box_pack_start(GTK_BOX(vbox2), table, FALSE, FALSE, 10);
    row = 0;

    gtk_widget_set_size_request(table, -1, 250);
    
    label = gtk_label_new("Scan path:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label,
                   0, 1, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);

    md->path = gtk_combo_box_new_text();
    gtk_combo_box_append_text(GTK_COMBO_BOX(md->path), "Regular");
    gtk_combo_box_append_text(GTK_COMBO_BOX(md->path), "Space filling");
    gtk_combo_box_append_text(GTK_COMBO_BOX(md->path), "Spiral");
    gtk_combo_box_append_text(GTK_COMBO_BOX(md->path), "Random");
    gtk_combo_box_append_text(GTK_COMBO_BOX(md->path), "Octave 1D");
    gtk_combo_box_append_text(GTK_COMBO_BOX(md->path), "Octave 2D");
    gtk_combo_box_append_text(GTK_COMBO_BOX(md->path), "Wichmann");
    gtk_combo_box_append_text(GTK_COMBO_BOX(md->path), "Refine");
    gtk_combo_box_append_text(GTK_COMBO_BOX(md->path), "Alternate Block");
    gtk_combo_box_set_active(GTK_COMBO_BOX(md->path), md->pathval);
    g_signal_connect( G_OBJECT(md->path), "changed",
                     G_CALLBACK(path_changed_cb), md);
    gtk_table_attach(GTK_TABLE(table), md->path,
	           1, 2, row, row+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);

    md->button_createscanpath = gtk_button_new_with_label("Create scan path");
    gtk_table_attach(GTK_TABLE(table), md->button_createscanpath,
        3, 4, row, row + 1, GTK_EXPAND | GTK_FILL, 0, 5, 5);
    g_signal_connect(md->button_createscanpath, "clicked", G_CALLBACK(createscanpath_cb), md);

    row++;

    md->xres_label = gtk_label_new("Pixel resolution:");
    gtk_misc_set_alignment(GTK_MISC(md->xres_label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), md->xres_label,
                   0, 1, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);

    md->xres_adj = gtk_adjustment_new(md->xres, 1, 100000, 1, 10, 0);
    md->xres_spin = gtk_spin_button_new(GTK_ADJUSTMENT(md->xres_adj), 1, 2);
    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(md->xres_spin), 0);
    gtk_table_attach(GTK_TABLE(table), md->xres_spin,
                   1, 2, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);


    md->yres_label = gtk_label_new("x");
    gtk_misc_set_alignment(GTK_MISC(md->yres_label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), md->yres_label,
                   2, 3, row, row+1, 0, 0, 5, 5);

    md->yres_adj = gtk_adjustment_new(md->yres, 1, 100000, 1, 10, 0);
    md->yres_spin = gtk_spin_button_new(GTK_ADJUSTMENT(md->yres_adj), 1, 2);
    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(md->yres_spin), 0);
    gtk_table_attach(GTK_TABLE(table), md->yres_spin,
                   3, 4, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    row++;

    label = gtk_label_new("Physical range [μm]:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label,
                   0, 1, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);

    md->xreal_adj = gtk_adjustment_new(md->xreal*1e6, 0.1, md->xrange*1e6, 1, 10, 0);
    md->xreal_spin = gtk_spin_button_new(GTK_ADJUSTMENT(md->xreal_adj), 1, 2);
    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(md->xreal_spin), 2);
    gtk_table_attach(GTK_TABLE(table), md->xreal_spin,
                   1, 2, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    label = gtk_label_new("x");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label,
                   2, 3, row, row+1, 0, 0, 5, 5);

    md->yreal_adj = gtk_adjustment_new(md->yreal*1e6, 0.1, md->yrange*1e6, 1, 10, 0);
    md->yreal_spin = gtk_spin_button_new(GTK_ADJUSTMENT(md->yreal_adj), 1, 2);
    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(md->yreal_spin), 2);
    gtk_table_attach(GTK_TABLE(table), md->yreal_spin,
                   3, 4, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    row++;

    label = gtk_label_new("Offset [μm]:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label,
                   0, 1, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);

    md->xoff_adj = gtk_adjustment_new(md->xoff*1e6, 0, md->xrange*1e6, 1, 10, 0);
    md->xoff_spin = gtk_spin_button_new(GTK_ADJUSTMENT(md->xoff_adj), 1, 2);
    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(md->xoff_spin), 2);
    gtk_table_attach(GTK_TABLE(table), md->xoff_spin,
                   1, 2, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    label = gtk_label_new(";");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label,
                   2, 3, row, row+1, 0, 0, 5, 5);

    md->yoff_adj = gtk_adjustment_new(md->yoff*1e6, 0, md->yrange*1e6, 1, 10, 0);
    md->yoff_spin = gtk_spin_button_new(GTK_ADJUSTMENT(md->yoff_adj), 1, 2);
    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(md->yoff_spin), 2);
    gtk_table_attach(GTK_TABLE(table), md->yoff_spin,
                   3, 4, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    row++;

    md->rotate_label = gtk_label_new("Rotate whole scan area [deg]:");
    gtk_misc_set_alignment(GTK_MISC(md->rotate_label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), md->rotate_label,
                   0, 3, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);

    md->rotate_adj = gtk_adjustment_new(md->rotate, -180, 180, 1, 10, 0);
    md->rotate_spin = gtk_spin_button_new(GTK_ADJUSTMENT(md->rotate_adj), 1, 2);
    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(md->rotate_spin), 1);
    gtk_table_attach(GTK_TABLE(table), md->rotate_spin,
                   3, 4, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);
    row++;



    md->nvals_label = gtk_label_new("Number of points:");
    gtk_misc_set_alignment(GTK_MISC(md->nvals_label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), md->nvals_label,
                   0, 3, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);

    md->nvals_adj = gtk_adjustment_new(md->nvals, 10, 1000000, 1, 100, 0);
    md->nvals_spin = gtk_spin_button_new(GTK_ADJUSTMENT(md->nvals_adj), 1, 2);
    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(md->nvals_spin), 0);
    gtk_table_attach(GTK_TABLE(table), md->nvals_spin,
                   3, 4, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    row++;

    md->order_label = gtk_label_new("Order:");
    gtk_misc_set_alignment(GTK_MISC(md->order_label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), md->order_label,
                   0, 3, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);

    md->order_adj = gtk_adjustment_new(md->order, 1, 1000, 1, 10, 0);
    md->order_spin = gtk_spin_button_new(GTK_ADJUSTMENT(md->order_adj), 1, 2);
    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(md->order_spin), 0);
    gtk_table_attach(GTK_TABLE(table), md->order_spin,
                   3, 4, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    row++;

    md->noctaves_label = gtk_label_new("Number of octaves:");
    gtk_misc_set_alignment(GTK_MISC(md->noctaves_label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), md->noctaves_label,
                   0, 3, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);

    md->noctaves_adj = gtk_adjustment_new(md->noctaves, 1, 10, 1, 10, 0);
    md->noctaves_spin = gtk_spin_button_new(GTK_ADJUSTMENT(md->noctaves_adj), 1, 2);
    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(md->noctaves_spin), 0);
    gtk_table_attach(GTK_TABLE(table), md->noctaves_spin,
                   3, 4, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    row++;

    md->nprofiles_label = gtk_label_new("Number of profiles:");
    gtk_misc_set_alignment(GTK_MISC(md->nprofiles_label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), md->nprofiles_label,
                   0, 3, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);

    md->nprofiles_adj = gtk_adjustment_new(md->nprofiles, 1, 10000, 1, 10, 0);
    md->nprofiles_spin = gtk_spin_button_new(GTK_ADJUSTMENT(md->nprofiles_adj), 1, 2);
    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(md->nprofiles_spin), 0);
    gtk_table_attach(GTK_TABLE(table), md->nprofiles_spin,
                   3, 4, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    row++;

    md->profile_check = gtk_check_button_new_with_mnemonic("repeat only single profile No.");
    gtk_table_attach(GTK_TABLE(table), md->profile_check,
                     0, 3, row, row+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);

    md->profile_adj = gtk_adjustment_new(md->profile, 0, 100000, 1, 10, 0);
    md->profile_spin = gtk_spin_button_new(GTK_ADJUSTMENT(md->profile_adj), 1, 2);
    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(md->profile_spin), 0);
    gtk_table_attach(GTK_TABLE(table), md->profile_spin,
                   3, 4, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);
    row++;


    md->pathlabel = gtk_label_new("No path created");
    gtk_misc_set_alignment(GTK_MISC(md->pathlabel), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), md->pathlabel,
                   0, 4, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);

    row++;          

    /***  in/out ***/
    page = gtk_vbox_new(FALSE, 0);
    gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
                               page,
                               gtk_label_new(("In/Out")));


    vbox2 = gtk_vbox_new(FALSE, 0);
    gtk_container_add(GTK_CONTAINER(page), vbox2);


    table = gtk_table_new(15, 5, FALSE);
    gtk_box_pack_start(GTK_BOX(vbox2), table, FALSE, FALSE, 10);
    row = 0;

    gtk_widget_set_size_request(table, -1, 250);

    label = gtk_label_new("Tip bias:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label,
                   0, 1, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);

    md->tipv_adj = gtk_adjustment_new(md->tipv, -10, 10, 0.001, 0.01, 0);
    md->tipv_spin = gtk_spin_button_new(GTK_ADJUSTMENT(md->tipv_adj), 1, 2);
    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(md->tipv_spin), 3);
    gtk_table_attach(GTK_TABLE(table), md->tipv_spin,
                   1, 2, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    label = gtk_label_new("V");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label,
                   2, 3, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);
    row++;

    label = gtk_label_new("Sample bias:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label,
                   0, 1, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);

    md->samplev_adj = gtk_adjustment_new(md->samplev, -10, 10, 0.001, 0.01, 0);
    md->samplev_spin = gtk_spin_button_new(GTK_ADJUSTMENT(md->samplev_adj), 1, 2);
    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(md->samplev_spin), 3);
    gtk_table_attach(GTK_TABLE(table), md->samplev_spin,
                   1, 2, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    label = gtk_label_new("V");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label,
                   2, 3, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);
    row++;

    label = gtk_label_new("Out 1 routing:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label,
                   0, 1, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);

    md->out1rule_combo
        = gwy_enum_combo_box_new(outs, G_N_ELEMENTS(outs),
                                 G_CALLBACK(out1rule_changed_cb),
                                 md, md->out1rule, TRUE);

    gtk_table_attach(GTK_TABLE(table), md->out1rule_combo,
                   1, 2, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    md->out1value_adj = gtk_adjustment_new(md->out1value, -10, 10, 0.001, 0.01, 0);
    md->out1value_spin = gtk_spin_button_new(GTK_ADJUSTMENT(md->out1value_adj), 1, 2);
    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(md->out1value_spin), 3);
    gtk_table_attach(GTK_TABLE(table), md->out1value_spin,
                   2, 3, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    label = gtk_label_new("V");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label,
                   3, 4, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);
    row++;

    label = gtk_label_new("Out 2 routing:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label,
                   0, 1, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);

    md->out2rule_combo
        = gwy_enum_combo_box_new(outs, G_N_ELEMENTS(outs),
                                 G_CALLBACK(out2rule_changed_cb),
                                 md, md->out2rule, TRUE);

    gtk_table_attach(GTK_TABLE(table), md->out2rule_combo,
                   1, 2, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    md->out2value_adj = gtk_adjustment_new(md->out2value, -10, 10, 0.001, 0.01, 0);
    md->out2value_spin = gtk_spin_button_new(GTK_ADJUSTMENT(md->out2value_adj), 1, 2);
    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(md->out2value_spin), 3);
    gtk_table_attach(GTK_TABLE(table), md->out2value_spin,
                   2, 3, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    label = gtk_label_new("V");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label,
                   3, 4, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);
    row++;


    /***  ramp ***/
    page = gtk_vbox_new(FALSE, 0);
    gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
                               page,
                               gtk_label_new(("Ramp")));


    vbox2 = gtk_vbox_new(FALSE, 0);
    gtk_container_add(GTK_CONTAINER(page), vbox2);


    table = gtk_table_new(15, 5, FALSE);
    gtk_box_pack_start(GTK_BOX(vbox2), table, FALSE, FALSE, 10);
    row = 0;

    gtk_widget_set_size_request(table, -1, 250);

    label = gtk_label_new("Ramp mode:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label,
                   0, 1, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);

    md->rampmode_combo
        = gwy_enum_combo_box_new(ramps, G_N_ELEMENTS(ramps),
                                 G_CALLBACK(rampmode_changed_cb),
                                 md, md->rampmode, TRUE);

    gtk_table_attach(GTK_TABLE(table), md->rampmode_combo,
	           1, 2, row, row+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);

    row++;


    label = gtk_label_new("From:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label,
                   0, 1, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);


    md->ramp_zfrom_adj = gtk_adjustment_new(md->ramp_zfrom, -6000, 6000, 1, 10, 0);
    md->ramp_zfrom_spin = gtk_spin_button_new(GTK_ADJUSTMENT(md->ramp_zfrom_adj), 1, 2);
    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(md->ramp_zfrom_spin), 3);
    gtk_table_attach(GTK_TABLE(table), md->ramp_zfrom_spin,
                   1, 2, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    md->rampqc_label = gtk_label_new("DAC,");
    gtk_misc_set_alignment(GTK_MISC(md->rampqc_label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), md->rampqc_label,
                   2, 3, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);

    label = gtk_label_new("to:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label,
                   3, 4, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);

    md->ramp_zto_adj = gtk_adjustment_new(md->ramp_zto, -6000, 6000, 1, 10, 0);
    md->ramp_zto_spin = gtk_spin_button_new(GTK_ADJUSTMENT(md->ramp_zto_adj), 1, 2);
    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(md->ramp_zto_spin), 3);
    gtk_table_attach(GTK_TABLE(table), md->ramp_zto_spin,
                   4, 5, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    md->rampq_label = gtk_label_new("DAC");
    gtk_misc_set_alignment(GTK_MISC(md->rampq_label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), md->rampq_label,
                   5, 6, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);

    row++;

    label = gtk_label_new("Points:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label,
                   0, 1, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);

    md->ramp_zres_adj = gtk_adjustment_new(md->ramp_zres, 10, 1024, 1, 10, 0);
    md->ramp_zres_spin = gtk_spin_button_new(GTK_ADJUSTMENT(md->ramp_zres_adj), 1, 2);
    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(md->ramp_zres_spin), 0);
    gtk_table_attach(GTK_TABLE(table), md->ramp_zres_spin,
                   1, 2, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);


    label = gtk_label_new("Ramp time:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label,
                   3, 4, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);

    md->ramp_time_adj = gtk_adjustment_new(md->ramp_time, 0, 1024, 1, 10, 0);
    md->ramp_time_spin = gtk_spin_button_new(GTK_ADJUSTMENT(md->ramp_time_adj), 1, 2);
    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(md->ramp_time_spin), 0);
    gtk_table_attach(GTK_TABLE(table), md->ramp_time_spin,
                   4, 5, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    label = gtk_label_new("ms");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label,
                   5, 6, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);

    row++;

    label = gtk_label_new("Start delay:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label,
                   0, 1, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);

    md->ramp_starttime_adj = gtk_adjustment_new(md->ramp_starttime, 0, 1024, 1, 10, 0);
    md->ramp_starttime_spin = gtk_spin_button_new(GTK_ADJUSTMENT(md->ramp_starttime_adj), 1, 2);
    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(md->ramp_starttime_spin), 0);
    gtk_table_attach(GTK_TABLE(table), md->ramp_starttime_spin,
                   1, 2, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    label = gtk_label_new("ms");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label,
                   2, 3, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);

    label = gtk_label_new("Peak delay:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label,
                   3, 4, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);

    md->ramp_peaktime_adj = gtk_adjustment_new(md->ramp_peaktime, 0, 1024, 1, 10, 0);
    md->ramp_peaktime_spin = gtk_spin_button_new(GTK_ADJUSTMENT(md->ramp_peaktime_adj), 1, 2);
    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(md->ramp_peaktime_spin), 0);
    gtk_table_attach(GTK_TABLE(table), md->ramp_peaktime_spin,
                   4, 5, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    label = gtk_label_new("ms");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label,
                   5, 6, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);


    row++;

    label = gtk_label_new("x position:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label,
                   0, 1, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);

    md->ramp_xpos_adj = gtk_adjustment_new(md->ramp_xpos/1e6, 0, md->xreal/1e6, 0.1, 1, 0);
    md->ramp_xpos_spin = gtk_spin_button_new(GTK_ADJUSTMENT(md->ramp_xpos_adj), 1, 2);
    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(md->ramp_xpos_spin), 3);
    gtk_table_attach(GTK_TABLE(table), md->ramp_xpos_spin,
                   1, 2, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    label = gtk_label_new("μm");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label,
                   2, 3, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);

    label = gtk_label_new("y position:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label,
                   3, 4, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);

    md->ramp_ypos_adj = gtk_adjustment_new(md->ramp_ypos/1e6, 0, md->yreal/1e6, 0.1, 1, 0);
    md->ramp_ypos_spin = gtk_spin_button_new(GTK_ADJUSTMENT(md->ramp_ypos_adj), 1, 2);
    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(md->ramp_ypos_spin), 3);
    gtk_table_attach(GTK_TABLE(table), md->ramp_ypos_spin,
                   4, 5, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    label = gtk_label_new("μm");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label,
                   5, 6, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);

    row++;
    md->ramp_do_check = gtk_check_button_new_with_mnemonic("run ramp at every scan pixel");
    gtk_table_attach(GTK_TABLE(table), md->ramp_do_check,
                     0, 3, row, row+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);

    row++;
    md->button_ramp = gtk_button_new_with_label("Run single ramp");
    gtk_table_attach(GTK_TABLE(table), md->button_ramp,
                     0, 6, row, row+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);

    g_signal_connect(md->button_ramp, "clicked",
                             G_CALLBACK(ramp_cb), md);


    /***  lift ***/
    page = gtk_vbox_new(FALSE, 0);
    gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
                               page,
                               gtk_label_new(("Lift")));


    vbox2 = gtk_vbox_new(FALSE, 0);
    gtk_container_add(GTK_CONTAINER(page), vbox2);


    table = gtk_table_new(15, 5, FALSE);
    gtk_box_pack_start(GTK_BOX(vbox2), table, FALSE, FALSE, 10);
    row = 0;

    gtk_widget_set_size_request(table, -1, 250);

    label = gtk_label_new("Lift mode:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label,
                   0, 1, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);

    md->liftmode_combo
        = gwy_enum_combo_box_new(lifts, G_N_ELEMENTS(lifts),
                                 G_CALLBACK(liftmode_changed_cb),
                                 md, md->liftmode, TRUE);

    gtk_table_attach(GTK_TABLE(table), md->liftmode_combo,
	           1, 2, row, row+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);

    label = gtk_label_new("Switch also:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label,
                   3, 4, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);

    md->liftswitch_combo
        = gwy_enum_combo_box_new(switches, G_N_ELEMENTS(switches),
                                 G_CALLBACK(liftswitch_changed_cb),
                                 md, md->lift_switch, TRUE);

    gtk_table_attach(GTK_TABLE(table), md->liftswitch_combo,
	           4, 5, row, row+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);


    row++;
  
    label = gtk_label_new("Number of lifts:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label,
                   0, 1, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);

    md->nlift_adj = gtk_adjustment_new(md->nlift, 0, 5, 1, 1, 0);
    md->nlift_spin = gtk_spin_button_new(GTK_ADJUSTMENT(md->nlift_adj), 1, 2);
    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(md->nlift_spin), 0);
    g_signal_connect(md->nlift_adj, "value-changed",
                     G_CALLBACK(nlift_changed_cb), md);
    gtk_table_attach(GTK_TABLE(table), md->nlift_spin,
                   1, 2, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    row++;


    label = gtk_label_new("Lift value 1:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label,
                   0, 1, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);

    md->lift1_height_adj = gtk_adjustment_new(md->lift1_height, 0, 5, 1, 1, 0);
    md->lift1_height_spin = gtk_spin_button_new(GTK_ADJUSTMENT(md->lift1_height_adj), 1, 2);
    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(md->lift1_height_spin), 3);
    g_signal_connect(md->lift1_height_adj, "value-changed",
                     G_CALLBACK(liftvalues_changed_cb), md);
    gtk_table_attach(GTK_TABLE(table), md->lift1_height_spin,
                   1, 2, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    md->lift1_label = gtk_label_new("μm,");
    gtk_misc_set_alignment(GTK_MISC(md->lift1_label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), md->lift1_label,
                   2, 3, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);

    label = gtk_label_new("switch to:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label,
                   3, 4, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);

    md->lift1_value_adj = gtk_adjustment_new(md->lift1_value, 0, 5, 1, 1, 0);
    md->lift1_value_spin = gtk_spin_button_new(GTK_ADJUSTMENT(md->lift1_value_adj), 1, 2);
    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(md->lift1_value_spin), 3);
    g_signal_connect(md->lift1_value_adj, "value-changed",
                     G_CALLBACK(liftvalues_changed_cb), md);
    gtk_table_attach(GTK_TABLE(table), md->lift1_value_spin,
                   4, 5, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    label = gtk_label_new("V");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label,
                   5, 6, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);
    row++;

    label = gtk_label_new("Lift value 2:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label,
                   0, 1, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);

    md->lift2_height_adj = gtk_adjustment_new(md->lift2_height, 0, 5, 1, 1, 0);
    md->lift2_height_spin = gtk_spin_button_new(GTK_ADJUSTMENT(md->lift2_height_adj), 1, 2);
    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(md->lift2_height_spin), 3);
    g_signal_connect(md->lift2_height_adj, "value-changed",
                     G_CALLBACK(liftvalues_changed_cb), md);
    gtk_table_attach(GTK_TABLE(table), md->lift2_height_spin,
                   1, 2, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    md->lift2_label = gtk_label_new("μm,");
    gtk_misc_set_alignment(GTK_MISC(md->lift2_label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), md->lift2_label,
                   2, 3, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);

    label = gtk_label_new("switch to:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label,
                   3, 4, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);

    md->lift2_value_adj = gtk_adjustment_new(md->lift2_value, 0, 5, 1, 1, 0);
    md->lift2_value_spin = gtk_spin_button_new(GTK_ADJUSTMENT(md->lift2_value_adj), 1, 2);
    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(md->lift2_value_spin), 3);
    g_signal_connect(md->lift2_value_adj, "value-changed",
                     G_CALLBACK(liftvalues_changed_cb), md);
    gtk_table_attach(GTK_TABLE(table), md->lift2_value_spin,
                   4, 5, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    label = gtk_label_new("V");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label,
                   5, 6, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);
    row++;


    label = gtk_label_new("Lift value 3:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label,
                   0, 1, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);

    md->lift3_height_adj = gtk_adjustment_new(md->lift3_height, 0, 5, 1, 1, 0);
    md->lift3_height_spin = gtk_spin_button_new(GTK_ADJUSTMENT(md->lift3_height_adj), 1, 2);
    g_signal_connect(md->lift3_height_adj, "value-changed",
                     G_CALLBACK(liftvalues_changed_cb), md);
    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(md->lift3_height_spin), 3);
    gtk_table_attach(GTK_TABLE(table), md->lift3_height_spin,
                   1, 2, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    md->lift3_label = gtk_label_new("μm,");
    gtk_misc_set_alignment(GTK_MISC(md->lift3_label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), md->lift3_label,
                   2, 3, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);

    label = gtk_label_new("switch to:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label,
                   3, 4, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);

    md->lift3_value_adj = gtk_adjustment_new(md->lift3_value, 0, 5, 1, 1, 0);
    md->lift3_value_spin = gtk_spin_button_new(GTK_ADJUSTMENT(md->lift3_value_adj), 1, 2);
    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(md->lift3_value_spin), 3);
    g_signal_connect(md->lift3_value_adj, "value-changed",
                     G_CALLBACK(liftvalues_changed_cb), md);
    gtk_table_attach(GTK_TABLE(table), md->lift3_value_spin,
                   4, 5, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    label = gtk_label_new("V");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label,
                   5, 6, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);
    row++;


    label = gtk_label_new("Lift value 4:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label,
                   0, 1, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);

    md->lift4_height_adj = gtk_adjustment_new(md->lift4_height, 0, 5, 1, 1, 0);
    md->lift4_height_spin = gtk_spin_button_new(GTK_ADJUSTMENT(md->lift4_height_adj), 1, 2);
    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(md->lift4_height_spin), 3);
    g_signal_connect(md->lift4_height_adj, "value-changed",
                     G_CALLBACK(liftvalues_changed_cb), md);
    gtk_table_attach(GTK_TABLE(table), md->lift4_height_spin,
                   1, 2, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    md->lift4_label = gtk_label_new("μm,");
    gtk_misc_set_alignment(GTK_MISC(md->lift4_label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), md->lift4_label,
                   2, 3, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);

    label = gtk_label_new("switch to:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label,
                   3, 4, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);

    md->lift4_value_adj = gtk_adjustment_new(md->lift4_value, 0, 5, 1, 1, 0);
    md->lift4_value_spin = gtk_spin_button_new(GTK_ADJUSTMENT(md->lift4_value_adj), 1, 2);
    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(md->lift4_value_spin), 3);
    g_signal_connect(md->lift4_value_adj, "value-changed",
                     G_CALLBACK(liftvalues_changed_cb), md);
    gtk_table_attach(GTK_TABLE(table), md->lift4_value_spin,
                   4, 5, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    label = gtk_label_new("V");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label,
                   5, 6, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);
    row++;


    label = gtk_label_new("Lift value 5:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label,
                   0, 1, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);

    md->lift5_height_adj = gtk_adjustment_new(md->lift5_height, 0, 5, 1, 1, 0);
    md->lift5_height_spin = gtk_spin_button_new(GTK_ADJUSTMENT(md->lift5_height_adj), 1, 2);
    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(md->lift5_height_spin), 3);
    g_signal_connect(md->lift5_height_adj, "value-changed",
                     G_CALLBACK(liftvalues_changed_cb), md);
    gtk_table_attach(GTK_TABLE(table), md->lift5_height_spin,
                   1, 2, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    md->lift5_label = gtk_label_new("μm,");
    gtk_misc_set_alignment(GTK_MISC(md->lift5_label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), md->lift5_label,
                   2, 3, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);

    label = gtk_label_new("switch to:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label,
                   3, 4, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);

    md->lift5_value_adj = gtk_adjustment_new(md->lift5_value, 0, 5, 1, 1, 0);
    md->lift5_value_spin = gtk_spin_button_new(GTK_ADJUSTMENT(md->lift5_value_adj), 1, 2);
    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(md->lift5_value_spin), 3);
    g_signal_connect(md->lift5_value_adj, "value-changed",
                     G_CALLBACK(liftvalues_changed_cb), md);
    gtk_table_attach(GTK_TABLE(table), md->lift5_value_spin,
                   4, 5, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    label = gtk_label_new("V");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label,
                   5, 6, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);
    row++;


    /***  kelvin ***/
    page = gtk_vbox_new(FALSE, 0);
    gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
                               page,
                               gtk_label_new(("AM KPFM")));


    vbox2 = gtk_vbox_new(FALSE, 0);
    gtk_container_add(GTK_CONTAINER(page), vbox2);


    table = gtk_table_new(15, 5, FALSE);
    gtk_box_pack_start(GTK_BOX(vbox2), table, FALSE, FALSE, 10);
    row = 0;

    gtk_widget_set_size_request(table, -1, 250);

    label = gtk_label_new("Voltage to:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label,
                   0, 1, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);

    md->kpfm_routing_combo
        = gwy_enum_combo_box_new(routs, G_N_ELEMENTS(routs),
                                 G_CALLBACK(kpfm_routing_changed_cb),
                                 md, md->kpfm_routing, TRUE);

    gtk_table_attach(GTK_TABLE(table), md->kpfm_routing_combo,
                   1, 2, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    row++;

    label = gtk_label_new("Frequency:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label,
                   0, 1, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);

    md->kpfm_frequency_adj = gtk_adjustment_new(md->kpfm_frequency, 0.1, 100, 0.001, 0.01, 0);
    md->kpfm_frequency_spin = gtk_spin_button_new(GTK_ADJUSTMENT(md->kpfm_frequency_adj), 1, 2);
    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(md->kpfm_frequency_spin), 3);
    gtk_table_attach(GTK_TABLE(table), md->kpfm_frequency_spin,
                   1, 2, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    label = gtk_label_new("kHz");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label,
                   2, 3, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);
    row++;

    label = gtk_label_new("Amplitude:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label,
                   0, 1, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);

    md->kpfm_amplitude_adj = gtk_adjustment_new(md->kpfm_amplitude, 0.1, 5, 0.001, 0.01, 0);
    md->kpfm_amplitude_spin = gtk_spin_button_new(GTK_ADJUSTMENT(md->kpfm_amplitude_adj), 1, 2);
    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(md->kpfm_amplitude_spin), 3);
    gtk_table_attach(GTK_TABLE(table), md->kpfm_amplitude_spin,
                   1, 2, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    label = gtk_label_new("V");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label,
                   2, 3, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);
    row++;

    label = gtk_label_new("Offset:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label,
                   0, 1, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);

    md->kpfm_offset_adj = gtk_adjustment_new(md->kpfm_offset, -5, 5, 0.001, 0.01, 0);
    md->kpfm_offset_spin = gtk_spin_button_new(GTK_ADJUSTMENT(md->kpfm_offset_adj), 1, 2);
    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(md->kpfm_offset_spin), 3);
    gtk_table_attach(GTK_TABLE(table), md->kpfm_offset_spin,
                   1, 2, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    label = gtk_label_new("V");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label,
                   2, 3, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);
    row++;

    label = gtk_label_new("Lock-in phase:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label,
                   0, 1, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);

    md->kpfm_phase_adj = gtk_adjustment_new(md->kpfm_phase, -90, 90, 0.001, 0.01, 0);
    md->kpfm_phase_spin = gtk_spin_button_new(GTK_ADJUSTMENT(md->kpfm_phase_adj), 1, 2);
    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(md->kpfm_phase_spin), 3);
    gtk_table_attach(GTK_TABLE(table), md->kpfm_phase_spin,
                   1, 2, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    label = gtk_label_new("deg");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label,
                   2, 3, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);
    row++;

    label = gtk_label_new("Bandwidth:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label,
                   0, 1, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);

    md->kpfm_bw_adj = gtk_adjustment_new(md->kpfm_bw, 0.001, 100, 0.001, 0.01, 0);
    md->kpfm_bw_spin = gtk_spin_button_new(GTK_ADJUSTMENT(md->kpfm_bw_adj), 1, 2);
    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(md->kpfm_bw_spin), 3);
    gtk_table_attach(GTK_TABLE(table), md->kpfm_bw_spin,
                   1, 2, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    label = gtk_label_new("kHz");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label,
                   2, 3, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);
    row++;


    /***  litography ***/
    page = gtk_vbox_new(FALSE, 0);
    gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
                               page,
                               gtk_label_new(("Litography")));


    vbox2 = gtk_vbox_new(FALSE, 0);
    gtk_container_add(GTK_CONTAINER(page), vbox2);


    table = gtk_table_new(15, 5, FALSE);
    gtk_box_pack_start(GTK_BOX(vbox2), table, FALSE, FALSE, 10);
    row = 0;

    gtk_widget_set_size_request(table, -1, 250);

    label = gtk_label_new("Routing:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label,
                   0, 1, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);

    md->litho_routing_combo
        = gwy_enum_combo_box_new(routs, G_N_ELEMENTS(routs),
                                 G_CALLBACK(litho_routing_changed_cb),
                                 md, md->litho_routing, TRUE);

    gtk_table_attach(GTK_TABLE(table), md->litho_routing_combo,
                   1, 2, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    row++;

    md->lithoview = gtk_text_view_new ();
    md->lithobuffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(md->lithoview));
    gtk_text_buffer_set_text(md->lithobuffer, "Enter here your litography data", -1);

    gtk_widget_set_size_request(md->lithoview, -1, 140);

    gtk_table_attach(GTK_TABLE(table), md->lithoview,
                   0, 2, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);
    row++;

    md->litho_button_load = gtk_button_new_with_label("Load");
    gtk_table_attach(GTK_TABLE(table), md->litho_button_load,
                     0, 1, row, row+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);

    g_signal_connect(md->litho_button_load, "clicked",
                             G_CALLBACK(litho_load_cb), md);

    md->litho_button_save = gtk_button_new_with_label("Save");
    gtk_table_attach(GTK_TABLE(table), md->litho_button_save,
                     1, 2, row, row+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);

    g_signal_connect(md->litho_button_save, "clicked",
                             G_CALLBACK(litho_save_cb), md);

    row++;

    md->litho_button_run = gtk_button_new_with_label("Run");
    gtk_table_attach(GTK_TABLE(table), md->litho_button_run,
                     0, 2, row, row+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);

    g_signal_connect(md->litho_button_run, "clicked",
                             G_CALLBACK(litho_run_cb), md);

    row++;
 

    /***  lua ***/
    page = gtk_vbox_new(FALSE, 0);
    gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
                               page,
                               gtk_label_new(("Lua script")));


    vbox2 = gtk_vbox_new(FALSE, 0);
    gtk_container_add(GTK_CONTAINER(page), vbox2);


    table = gtk_table_new(15, 5, FALSE);
    gtk_box_pack_start(GTK_BOX(vbox2), table, FALSE, FALSE, 10);
    row = 0;

    gtk_widget_set_size_request(table, -1, 250);

    label = gtk_label_new("Allocated data size:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label,
                   0, 1, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);

    md->lua_size_adj = gtk_adjustment_new(md->lua_size, 1, 10000000, 10, 1000, 0);
    md->lua_size_spin = gtk_spin_button_new(GTK_ADJUSTMENT(md->lua_size_adj), 1, 2);
    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(md->lua_size_spin), 0);
    gtk_table_attach(GTK_TABLE(table), md->lua_size_spin,
                   1, 2, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    row++;

    md->luaview = gtk_text_view_new ();
    md->luabuffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(md->luaview));
    gtk_text_buffer_set_text(md->luabuffer, "local scan = {}\n"
                                            "\n"
                                            "function scan.runit()\n"
                                            "  gws_clear(p)\n"
                                            "  print(gws_get_scan_param(p, \"prase\"))\n "
                                            "  for x = 0, 10e-6, 0.1e-6\n"
                                            "  do\n"
                                            "     for y = 0, 10e-6, 0.1e-6\n"
                                            "     do\n"
                                            "            gws_move_to(p, x, y, 0)\n"
                                            "            gws_store_point(p, x, y, 0)\n"
                                            "            if gws_check_if_stopped(p)==1 then\n"
                                            "               print(\"stopped on lua side\")\n"
                                            "               return scan\n"
                                            "               end\n"
                                            "     end\n"
                                            "  end\n"
                                            "  print(\"completed on lua side\")\n"
                                            "end\n"
                                            "return scan\n", -1);

    gtk_widget_set_size_request(md->luaview, -1, 180);

    gtk_table_attach(GTK_TABLE(table), md->luaview,
                   0, 2, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);
    row++;

    md->lua_button_load = gtk_button_new_with_label("Load");
    gtk_table_attach(GTK_TABLE(table), md->lua_button_load,
                     0, 1, row, row+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);

    g_signal_connect(md->lua_button_load, "clicked",
                             G_CALLBACK(lua_load_cb), md);

    md->lua_button_save = gtk_button_new_with_label("Save");
    gtk_table_attach(GTK_TABLE(table), md->lua_button_save,
                     1, 2, row, row+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);

    g_signal_connect(md->lua_button_save, "clicked",
                             G_CALLBACK(lua_save_cb), md);

    row++;
 
    /****  General settings  *****************************************************************************************/
    frame = gtk_frame_new("General scan settings");
    gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_ETCHED_IN);
    gtk_box_pack_start(GTK_BOX(vboxleft), frame, FALSE, TRUE, 0);

    vbox2 = gtk_vbox_new(FALSE, 0);
    gtk_container_add(GTK_CONTAINER(frame), vbox2);

    table = gtk_table_new(12, 7, FALSE);
    gtk_box_pack_start(GTK_BOX(vbox2), table, FALSE, TRUE, 0);
    row = 0;

    label = gtk_label_new("Preview step:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label, 0, 2, row, row + 1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    md->preview_step_adj = gtk_adjustment_new(md->preview_step, 10, 100000, 1, 10, 0);
    md->preview_step_spin = gtk_spin_button_new(GTK_ADJUSTMENT(md->preview_step_adj), 1, 2);
    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(md->preview_step_spin), 0);
    gtk_table_attach(GTK_TABLE(table), md->preview_step_spin, 2, 3, row, row + 1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    label = gtk_label_new("points");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label, 3, 4, row, row + 1, GTK_EXPAND | GTK_FILL, 0, 5, 1);
    row++;


    label = gtk_label_new("Wait in each point:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label,
                   0, 2, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 5);

    md->wait_adj = gtk_adjustment_new(md->wait, 0, 1000, 1, 5, 0);
    md->wait_spin = gtk_spin_button_new(GTK_ADJUSTMENT(md->wait_adj), 1, 2);
    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(md->wait_spin), 0);
    g_signal_connect(md->wait_adj, "value-changed",
                     G_CALLBACK(wait_changed_cb), md);
    gtk_table_attach(GTK_TABLE(table), md->wait_spin,
                   2, 3, row, row+1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    label = gtk_label_new("ms");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label, 3, 4, row, row + 1, GTK_EXPAND | GTK_FILL, 0, 5, 1);
    row++;

    row++;

    label = gtk_label_new("Speed:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label,
        0, 2, row, row + 1, GTK_EXPAND | GTK_FILL, 0, 5, 5);

    md->speed_adj = gtk_adjustment_new(md->speed, 0.01, 100, 0.01, 1, 0);
    md->speed_spin = gtk_spin_button_new(GTK_ADJUSTMENT(md->speed_adj), 1, 2);
    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(md->speed_spin), 2);
    g_signal_connect(md->speed_adj, "value-changed",
                     G_CALLBACK(speed_changed_cb), md);
    gtk_table_attach(GTK_TABLE(table), md->speed_spin,
                     2, 3, row, row + 1, GTK_EXPAND | GTK_FILL, 0, 5, 1);

    label = gtk_label_new("μm/s");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label, 3, 4, row, row + 1, GTK_EXPAND | GTK_FILL, 0, 5, 1);
    row++;

    row++;

 
    hbox2 = gtk_hbox_new(TRUE, 0);

    align = gtk_alignment_new(0.0, 1.0, 0.0, 0.0);
    gtk_box_pack_start(GTK_BOX(vbox2), align, FALSE, FALSE, 0);

    gtk_box_pack_start(GTK_BOX(vbox2), hbox2, FALSE, FALSE, 0);            

    md->button_scan = gtk_button_new_with_label("Scan");
    gtk_box_pack_start(GTK_BOX(hbox2), md->button_scan, TRUE, TRUE, 0);
    g_signal_connect(md->button_scan, "clicked", G_CALLBACK(scan_cb), md);    

    md->button_pause = gtk_button_new_with_label("Pause");
    gtk_box_pack_start(GTK_BOX(hbox2), md->button_pause, TRUE, TRUE, 0);
    g_signal_connect(md->button_pause, "clicked", G_CALLBACK(pause_cb), md);    

    
    /****  system log  **************************************************************************************************/
    frame = gtk_frame_new("System log");
    gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_ETCHED_IN);
    gtk_box_pack_start(GTK_BOX(vboxleft), frame, TRUE, TRUE, 0);

    scroll = gtk_scrolled_window_new(NULL, NULL);
    gtk_container_add(GTK_CONTAINER(frame), scroll);

    md->tblog = gtk_text_buffer_new(NULL);
    md->viewlog = gtk_text_view_new_with_buffer(md->tblog);
    gtk_container_add(GTK_CONTAINER(scroll), md->viewlog);


    /****  four data views  ****/
    table = gtk_table_new(2, 2, TRUE);
    gtk_box_pack_start(GTK_BOX(vboxright), table, TRUE, TRUE, 0);

    md->view = (GwsChanview **)g_malloc(4*sizeof(GwsChanview*));      

    md->view[0] = gws_chanview_new(md->container, md->rcontainer, 0, md);     //FIXME add also rcontainer 
    gtk_table_attach_defaults(GTK_TABLE(table), md->view[0]->vbox, 0, 1, 0, 1);

    md->view[1] = gws_chanview_new(md->container, md->rcontainer, 1, md);
    gtk_table_attach_defaults(GTK_TABLE(table), md->view[1]->vbox, 0, 1, 1, 2);

    md->view[2] = gws_chanview_new(md->container, md->rcontainer, 2, md);
    gtk_table_attach_defaults(GTK_TABLE(table), md->view[2]->vbox, 1, 2, 0, 1);

    md->view[3] = gws_chanview_new(md->container, md->rcontainer, 3, md);
    gtk_table_attach_defaults(GTK_TABLE(table), md->view[3]->vbox, 1, 2, 1, 2);


    /*two graphs*/
    hbox2 = gtk_hbox_new(TRUE, 0);
    gtk_box_pack_start(GTK_BOX(vboxright), hbox2, TRUE, TRUE, 0);

    md->graph = (GwsChangraph **)g_malloc(2*sizeof(GwsChangraph*));      

    md->graph[0] = gws_changraph_new(md->container, md->rcontainer, 0, md);   //FIXME add also rcontainer
    gtk_box_pack_start(GTK_BOX(hbox2), md->graph[0]->vbox, TRUE, TRUE, 0);

    md->graph[1] = gws_changraph_new(md->container, md->rcontainer, 1, md);
    gtk_box_pack_start(GTK_BOX(hbox2), md->graph[1]->vbox, TRUE, TRUE, 0);

    gws_changraph_change_state(md->graph[0], CHGR_STATE_FREERUNNING);
    gws_changraph_change_state(md->graph[1], CHGR_STATE_FREERUNNING);


    /****  Statusbar measure   *****/
    md->statusbar_measure = gwy_statusbar_new();
    gtk_box_pack_start(GTK_BOX (vbox), md->statusbar_measure, FALSE, FALSE, 0);
    gwy_statusbar_set_markup(GWY_STATUSBAR(md->statusbar_measure), "Ready");

    /****  Statusbar scan   *****/
    md->statusbar_scan = gwy_statusbar_new();
    gtk_box_pack_start(GTK_BOX(vbox), md->statusbar_scan, FALSE, FALSE, 0);
    gwy_statusbar_set_markup(GWY_STATUSBAR(md->statusbar_scan), "Ready");




    g_signal_connect_swapped(G_OBJECT(md->xres_spin), "changed", 
			    G_CALLBACK(scan_widgets_read), md);
    g_signal_connect_swapped(G_OBJECT(md->yres_spin), "changed", 
			    G_CALLBACK(scan_widgets_read), md);
    g_signal_connect_swapped(G_OBJECT(md->xreal_spin), "changed", 
			    G_CALLBACK(scan_widgets_read), md);
    g_signal_connect_swapped(G_OBJECT(md->yreal_spin), "changed", 
			    G_CALLBACK(scan_widgets_read), md);
    g_signal_connect_swapped(G_OBJECT(md->rotate_spin), "changed", 
			    G_CALLBACK(scan_widgets_read), md);
    g_signal_connect_swapped(G_OBJECT(md->nvals_spin), "changed", 
			    G_CALLBACK(scan_widgets_read), md);
    g_signal_connect_swapped(G_OBJECT(md->order_spin), "changed", 
			    G_CALLBACK(scan_widgets_read), md);
    g_signal_connect_swapped(G_OBJECT(md->noctaves_spin), "changed", 
			    G_CALLBACK(scan_widgets_read), md);
    g_signal_connect_swapped(G_OBJECT(md->nprofiles_spin), "changed", 
			    G_CALLBACK(scan_widgets_read), md);
    g_signal_connect_swapped(G_OBJECT(md->preview_step_spin), "changed",
                            G_CALLBACK(scan_widgets_read), md);
    g_signal_connect_swapped(G_OBJECT(md->ramp_xpos_spin), "changed", 
			    G_CALLBACK(scan_widgets_read), md);
    g_signal_connect_swapped(G_OBJECT(md->ramp_ypos_spin), "changed", 
			    G_CALLBACK(scan_widgets_read), md);
  
    md->in_setup = 0;

    gws_chanview_enable_reverse(md->view[0], md->c_sc_reverse);
    gws_chanview_enable_reverse(md->view[1], md->c_sc_reverse);
    gws_chanview_enable_reverse(md->view[2], md->c_sc_reverse);
    gws_chanview_enable_reverse(md->view[3], md->c_sc_reverse);

    mode_changed(GTK_COMBO_BOX(md->mode_combo), md); 
    rampmode_changed_cb(GTK_COMBO_BOX(md->rampmode_combo), md); 
    nlift_changed_cb(md->nlift_adj, md);
    liftmode_changed_cb(GTK_COMBO_BOX(md->liftmode_combo), md); 
    liftswitch_changed_cb(GTK_COMBO_BOX(md->liftswitch_combo), md); 
    out1rule_changed_cb(GTK_COMBO_BOX(md->out1rule_combo), md);
    out2rule_changed_cb(GTK_COMBO_BOX(md->out2rule_combo), md);
    kpfm_routing_changed_cb(GTK_COMBO_BOX(md->kpfm_routing_combo), md);

    phaselimit_changed_cb(GTK_COMBO_BOX(md->phaselimit_combo), md);
    fmlimit_changed_cb(GTK_COMBO_BOX(md->fmlimit_combo), md);
    phasefilter_toggled_cb(GTK_TOGGLE_BUTTON(md->phasefilter_check), md);
    amfm_toggled_cb(GTK_TOGGLE_BUTTON(md->amfm_check), md);

    set_filter(md, 0, md->filter1);
    set_filter(md, 1, md->filter2);
    set_phaseshift(md, md->phaseshift1);

    gtk_widget_show_all(window);
    show_hide_widgets(md);

    path_changed_cb(GTK_COMBO_BOX(md->path), md);
    scan_widgets_read(md);

    set_feedback(md, FALSE);
    set_fmfeedback(md, FALSE);
    set_mode(md, md->mode);
    set_setpoint(md, md->threshold);

    set_ranges(md);

    gws_changraph_set_running(md->graph[0], TRUE);
    gws_changraph_set_running(md->graph[1], TRUE);
    gws_chanview_set_running(md->view[0], TRUE);
    gws_chanview_set_running(md->view[1], TRUE);
    gws_chanview_set_running(md->view[2], TRUE);
    gws_chanview_set_running(md->view[3], TRUE);

    g_timeout_add(100, valsupdate, md);

    writelog(md, "GUI setup.");


    return window;
}



